-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

-- Caster Types: 0 = Memorization, 1 = Spontaneous, 2 = Mana
-- Mode: 0 = Standard, 1 = Preparation, 2 = Combat

local bInitialized = false;
	
function onInit()
	bInitialized = true;

	onCasterTypeChanged();

	local node = getDatabaseNode();
	DB.addHandler(DB.getPath(node, "spellset.set1.spontaneous"), "onUpdate", onCasterTypeChanged);
	DB.addHandler(DB.getPath(node, "spellset.set1.mana.total"), "onUpdate", onSpellCounterUpdate);
	DB.addHandler(DB.getPath(node, "spellset.set1.mana.used"), "onUpdate", onSpellCounterUpdate);
end

function onClose()
	local node = getDatabaseNode();
	DB.removeHandler(DB.getPath(node, "spellset.set1.spontaneous"), "onUpdate", onCasterTypeChanged);
	DB.removeHandler(DB.getPath(node, "spellset.set1.mana.total"), "onUpdate", onSpellCounterUpdate);
	DB.removeHandler(DB.getPath(node, "spellset.set1.mana.used"), "onUpdate", onSpellCounterUpdate);
end

function update()
	if minisheet then
		return;
	end
	if not bInitialized then
		return;
	end

	local bEditMode = (actions_iedit.getValue() == 1);
	for _,w in ipairs(levels.getWindows()) do
		w.update(bEditMode);
	end
end

function registerMenuItems()
	resetMenuItems();
	
	if DB.getValue(getDatabaseNode(), "spontaneous", 0) == 0 then
		registerMenuItem(Interface.getString("menu_resetspells"), "pointer_circle", 3);
	end
end

function getActorType()
	return "pc";
end	

function onMenuSelection(selection, subselection)
	if selection == 3 then
		local nodeCaster = getDatabaseNode().getChild("...");
		SpellManager.resetPrepared(nodeCaster);
	end
end

function isInitialized()
	return bInitialized;
end

function getSheetMode()
	if minisheet then
		return 2;
	end
	
	return DB.getValue(getDatabaseNode(), "castingmode", 0);
end

function onModeChanged()
	updateSpellCounters();
end

function updateSpellCounters()
	if minisheet then
		for _,v in pairs(levels.getWindows()) do
			v.onSpellCounterUpdate();
		end
	else
		for _,v in pairs(levels.getWindows()) do
			v.onSpellCounterUpdate();
		end
	end
end

function updateControl(sControl, bShow)
	local bLocalShow = bShow;
	
	if self[sControl] then
		self[sControl].setVisible(bLocalShow);
	else
		bLocalShow = false;
	end

	if self[sControl .. "_label"] then
		self[sControl .. "_label"].setVisible(bLocalShow);
	end
end

function onCasterTypeChanged()
	if minisheet then
		onSpellCounterUpdate();
		return;
	end
	
	local bManaCaster = (DB.getValue(getDatabaseNode(), "spellset.set1.spontaneous", 0) == 2);
	
	updateControl("availablelevel0", not bManaCaster);
	updateControl("availablelevel1", not bManaCaster);
	updateControl("availablelevel2", not bManaCaster);
	updateControl("availablelevel3", not bManaCaster);
	updateControl("availablelevel4", not bManaCaster);
	updateControl("availablelevel5", not bManaCaster);
	updateControl("availablelevel6", not bManaCaster);
	updateControl("availablelevel7", not bManaCaster);
	updateControl("availablelevel8", not bManaCaster);
	updateControl("availablelevel9", not bManaCaster);
	
	mana_label.setVisible(bManaCaster);
	mana_label2.setVisible(bManaCaster);
	updateControl("manatotal", bManaCaster);
	updateControl("manaused", bManaCaster);
	
	onSpellCounterUpdate();
	
	registerMenuItems();
end

function onSpellCounterUpdate()
	if not isInitialized() then
		return;
	end
	
	SpellManager.updateSpellClassCounts(DB.getChild(getDatabaseNode(), "spellset.set1"));

	updateSpellView();
	
	performFilter();
end

function updateSpellView()
	local nodeSpellClass = getDatabaseNode().getChild("spellset.set1");

	local nSheetMode = getSheetMode();
	local nCasterType = DB.getValue(nodeSpellClass, "spontaneous", 0);

	local bLevelShow, nodeLevel, nAvailable, nTotalCast, nTotalPrepared, nMaxPrepared, nSpells;
	local bSpellShow, nodeSpell, nCast, nPrepared, nPointCost;

	local nPP = DB.getValue(nodeSpellClass, "mana.total", 0);
	local nPPUsed = DB.getValue(nodeSpellClass, "mana.used", 0);
	
	for kLevel, vLevel in pairs(levels.getWindows()) do
		bLevelShow = false;

		nAvailable = 0;
		nodeLevel = vLevel.getDatabaseNode();
		if nodeLevel then
			nAvailable = DB.getValue(nodeSpellClass, "spellsavailable.available" .. nodeLevel.getName(), 0);
		end
		
		nSpells = 0;
		nTotalCast = DB.getValue(nodeLevel, "totalcast", 0);
		nTotalPrepared = DB.getValue(nodeLevel, "totalprepared", 0);
		nMaxPrepared = DB.getValue(nodeLevel, "maxprepared", 0);

		if nCasterType == 2 then
			for _,vSpell in pairs(vLevel.spells.getWindows()) do
				nodeSpell = vSpell.getDatabaseNode();
				nSpells = nSpells + 1;
				
				nPointCost = DB.getValue(nodeSpell, "cost", 0);
				
				if nSheetMode == 2 then
					bSpellShow = (nPointCost <= (nPP - nPPUsed));
				else
					bSpellShow = true;
				end
				vSpell.setFilter(bSpellShow);
				bLevelShow = bLevelShow or bSpellShow;

				vSpell.usepower.setVisible(true);
				vSpell.cost.setVisible(true);
				vSpell.counter.setVisible(false);
				vSpell.usespacer.setVisible(false);
			end
		
			if nSheetMode == 2 then
				bLevelShow = bLevelShow and (nSpells > 0);
			end
		else
			-- Update spell counter objects and spell visibility
			for _,vSpell in pairs(vLevel.spells.getWindows()) do
				nodeSpell = vSpell.getDatabaseNode();
				nSpells = nSpells + 1;
				
				nCast = DB.getValue(nodeSpell, "cast", 0);
				nPrepared = DB.getValue(nodeSpell, "prepared", 0);
				
				if nCasterType == 1 or nSheetMode ~= 2 then
					bSpellShow = true;
				else
					bSpellShow = (nCast < nPrepared);
				end
				bLevelShow = bLevelShow or bSpellShow;
				vSpell.setFilter(bSpellShow);

				vSpell.usepower.setVisible(false);
				vSpell.cost.setVisible(false);
				vSpell.counter.setVisible(true);
				vSpell.counter.update(nSheetMode, (nCasterType == 1), nAvailable, nTotalCast, nTotalPrepared, nMaxPrepared);
				if (nSheetMode == 1 or nCasterType == 1) then
					vSpell.usespacer.setVisible(nAvailable == 0);
				else
					vSpell.usespacer.setVisible(false);
				end
			end
			
			-- Determine level visibility
			if nSheetMode == 2 then
				bLevelShow = bLevelShow and (nTotalCast < nAvailable) and (nAvailable > 0) and (nSpells > 0);
			else
				bLevelShow = (nAvailable > 0);
			end
		end
		vLevel.setFilter(bLevelShow);

		if not minisheet then
			-- Set level statistics label
			local sStats = "";
			if (nCasterType ~= 2) and (nAvailable > 0) and (nSpells > 0) then
				if (nCasterType == 1) then
					sStats = "Cast:  " .. nTotalCast .. " / " .. nAvailable;
				else
					sStats = "Cast:  " .. nTotalCast .. " / " .. nTotalPrepared;
					if nTotalPrepared < nAvailable then
						sStats = sStats .. "    Prepared:  " .. nTotalPrepared .. " / " .. nAvailable;
					end
				end
			end
			vLevel.stats.setValue(sStats);
		end
	end
end

function performFilter()
	for _,vLevel in pairs(levels.getWindows()) do
		vLevel.spells.applyFilter();
	end
	levels.applyFilter();
end

function onDrop(x, y, draginfo)
	if draginfo.isType("spelldescwithlevel") then
		local nSourceLevel = draginfo.getNumberData();
		local nodeWin = getDatabaseNode();
		local nodeTargetLevelSpells = nodeWin.getChild("spellset.set1.levels.level" .. nSourceLevel .. ".spells");
		
		local nodeNew = SpellManager.addSpell(draginfo.getDatabaseNode(), nodeTargetLevelSpells);
		if nodeNew then
			-- Make sure new spell is visible
			local nodeTargetLevel = nodeWin.getChild("levels.level" .. nSourceLevel);
			for _,vWin in pairs(levels.getWindows()) do
				if vWin.getDatabaseNode() == nodeTargetLevel then
					vWin.spells.setVisible(true);
					break;
				end
			end
			DB.setValue(nodeWin, "castingmode", "number", 0);
		end
		return true;
		
	elseif draginfo.isType("shortcut") then
		local sClass = draginfo.getShortcutData();
		
		if sClass == "spell" or sClass == "spelldesc" then
			ChatManager.Message("Unable to determine level for new spell, please drop within a level spell list.");
			return true;
		end
	end
end