-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	update();
	DB.addHandler(DB.getPath(getDatabaseNode(), "itemtype"), "onUpdate", update);
end

function onClose()
	DB.removeHandler(DB.getPath(getDatabaseNode(), "itemtype"), "onUpdate", update);
end

function updateControl(sControl, bReadOnly, bID)
	if not self[sControl] then
		return false;
	end
		
	if not bID then
		return self[sControl].update(bReadOnly, true);
	end
	
	return self[sControl].update(bReadOnly);
end

function update()
	local nodeRecord = getDatabaseNode();
	local bReadOnly = WindowManager.getReadOnlyState(nodeRecord);
	local bID, bOptionID = LibraryData.getIDState("item", nodeRecord);
	
	local sType = itemtype.getValue();
	local bWeapon = (sType == "weapon");
	local bArmor = (sType == "armor");
	local bContainer = (sType == "container");
	
	local bSection1 = false;
	if bOptionID and User.isHost() then
		if updateControl("nonid_name", bReadOnly, true) then bSection1 = true; end;
	else
		updateControl("nonid_name", false);
	end
	if bOptionID and (User.isHost() or not bID) then
		if updateControl("nonidentified", bReadOnly, true) then bSection1 = true; end;
	else
		updateControl("nonidentified", false);
	end

	local bSection2 = false;
	if updateControl("value", bReadOnly, bID) then bSection2 = true; end
	if updateControl("weightlbs", bReadOnly, bID) then bSection2 = true; end
	if updateControl("ev", bReadOnly, bID) then bSection2 = true; end
	if updateControl("evcapacity", bReadOnly, bID and bContainer) then bSection2 = true; end
	
	local bSection3 = false;
	if updateControl("subtype", bReadOnly, bID and (bWeapon or bArmor)) then bSection3 = true; end
	if updateControl("damage", bReadOnly, bID and bWeapon) then bSection3 = true; end
	if updateControl("range", bReadOnly, bID and bWeapon) then bSection3 = true; end
	if updateControl("wield", bReadOnly, bID and bWeapon) then bSection3 = true; end
	if updateControl("ac", bReadOnly, bID and bArmor) then bSection3 = true; end

	local bSection4 = bID;
	description.setVisible(bID);
	description.setReadOnly(bReadOnly);
	
	divider.setVisible(bSection1 and bSection2);
	divider2.setVisible((bSection1 or bSection2) and bSection3);
	divider3.setVisible((bSection1 or bSection2 or bSection3) and bSection4);
end
