-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	ItemManager.setCustomCharAdd(onCharItemAdd);
	ItemManager.setCustomCharRemove(onCharItemDelete);
	initWeaponIDTracking();
end

--
-- ITEM/FOCUS MANAGEMENT
--

function onCharItemAdd(nodeItem)
	DB.setValue(nodeItem, "carried", "number", 1);

	addToWeaponDB(nodeItem);
end

function onCharItemDelete(nodeItem)
	removeFromWeaponDB(nodeItem);
end

local sEncColor = "006600";
function getEncumbranceColor()
	return sEncColor;
end

function updateEncumbrance(nodeChar)
	local nEncTotal = 0;

	local aItemByName = {};
	local aLocation = {};
	
	local nCount, nWeight, sLoc;
	for _,vNode in pairs(DB.getChildren(nodeChar, "inventorylist")) do
		local nCarried = DB.getValue(vNode, "carried", 0);
		if nCarried > 0 then
			nCount = DB.getValue(vNode, "count", 0);
			if nCount < 1 then
				nCount = 1;
			end
			nWeight = DB.getValue(vNode, "ev", 0);
			
			sLoc = DB.getValue(vNode, "location", ""):lower();
			if sLoc ~= "" then
				if not aLocation[sLoc] then
					aLocation[sLoc] = {};
				end
				table.insert(aLocation[sLoc], { nCount = nCount, nWeight = nWeight });
			end
			
			sName = DB.getValue(vNode, "name", ""):lower();
			aItemByName[sName] = vNode;

			nEncTotal = nEncTotal + (nCount * nWeight);
		end
	end
	
	for k,v in pairs(aLocation) do
		if aItemByName[k] then
			local nEVCapacity = DB.getValue(aItemByName[k], "evcapacity", 0);
			if nEVCapacity > 0 then
				local nItemsInContainer = 0;
				for _,v2 in ipairs(v) do
					if v2.nWeight < nEVCapacity then
						nRemove = math.min(v2.nCount, nEVCapacity - nItemsInContainer);
						nItemsInContainer = nItemsInContainer + nRemove;
						nEncTotal = nEncTotal - (nRemove * v2.nWeight);
					end
				end
			end
		end
	end

	DB.setValue(nodeChar, "encumbrance.load", "number", nEncTotal);
	
	local nEVRating = DB.getValue(nodeChar, "encumbrance.rating", 10);
	local nEncState = nEncTotal / nEVRating;
	if nEncState > 3 then
		sEncColor = "E56717";
		DB.setValue(nodeChar, "encumbrance.state", "string", "Overburdened");
	elseif nEncState > 1 then
		sEncColor = "408000";
		DB.setValue(nodeChar, "encumbrance.state", "string", "Burdened");
	else
		sEncColor = "006600";
		DB.setValue(nodeChar, "encumbrance.state", "string", "Unburdened");
	end
end

--
-- WEAPON MANAGEMENT
--

function removeFromWeaponDB(nodeItem)
	if not nodeItem then
		return false;
	end
	
	-- Check to see if any of the weapon nodes linked to this item node should be deleted
	local sItemNode = nodeItem.getNodeName();
	local sItemNode2 = "....inventorylist." .. nodeItem.getName();
	local bFound = false;
	for _,v in pairs(DB.getChildren(nodeItem, "...weaponlist")) do
		local sClass, sRecord = DB.getValue(v, "shortcut", "", "");
		if sRecord == sItemNode or sRecord == sItemNode2 then
			bFound = true;
			v.delete();
		end
	end

	return bFound;
end

function addToWeaponDB(nodeItem)
	-- Parameter validation
	if DB.getValue(nodeItem, "itemtype", ""):lower() ~= "weapon" then
		return;
	end
	
	-- Get the weapon list we are going to add to
	local nodeChar = nodeItem.getChild("...");
	local nodeWeapons = nodeChar.createChild("weaponlist");
	if not nodeWeapons then
		return nil;
	end
	
	-- Set new weapons as equipped
	DB.setValue(nodeItem, "carried", "number", 2);

	-- Determine identification
	local nItemID = 0;
	if LibraryData.getIDState("item", nodeItem, true) then
		nItemID = 1;
	end
	
	-- Keep track of which weapon IDs were added
	local sItemNode = nodeItem.getNodeName();
	
	-- Grab some information from the source node to populate the new weapon entries
	local sName;
	if nItemID == 1 then
		sName = DB.getValue(nodeItem, "name", "");
	else
		sName = DB.getValue(nodeItem, "nonid_name", "");
		if sName == "" then
			sName = Interface.getString("item_unidentified");
		end
		sName = "** " .. sName .. " **";
	end

	-- Determine the weapon type
	local sSubType = DB.getValue(nodeItem, "subtype", ""):lower();
	local nRange = DB.getValue(nodeItem, "range", 0);

	local bMelee = (sSubType == "melee") or (nRange == 0);
	local bThrown = (bMelee and (nRange > 0)) or (sSubType == "thrown");
	local bRanged = (nRange > 0) and not bThrown;

	local sDamage = DB.getValue(nodeItem, "damage", "");
	local aDamageDice, nDamage = StringManager.convertStringToDice(sDamage);
	
	-- Create melee entry
	if bMelee then
		local nodeWeapon = nodeWeapons.createChild();
		if nodeWeapon then
			DB.setValue(nodeWeapon, "isidentified", "number", nItemID);
			DB.setValue(nodeWeapon, "name", "string", sName);
			
			DB.setValue(nodeWeapon, "damagedice", "dice", aDamageDice);
			DB.setValue(nodeWeapon, "damagebonus", "number", nDamage);

			DB.setValue(nodeWeapon, "shortcut", "windowreference", "item", "....inventorylist." .. nodeItem.getName());
		end
	end
	
	-- Create ranged entry
	if bRanged then
		local nodeWeapon = nodeWeapons.createChild();
		if nodeWeapon then
			DB.setValue(nodeWeapon, "isidentified", "number", nItemID);
			DB.setValue(nodeWeapon, "name", "string", sName);
			DB.setValue(nodeWeapon, "type", "number", 1);
			
			DB.setValue(nodeWeapon, "damagedice", "dice", aDamageDice);
			DB.setValue(nodeWeapon, "damagebonus", "number", nDamage);
		
			DB.setValue(nodeWeapon, "rangeincrement", "number", nRange);

			DB.setValue(nodeWeapon, "shortcut", "windowreference", "item", "....inventorylist." .. nodeItem.getName());
		end
	end
	
	-- Create thrown entry
	if bThrown then
		local nodeWeapon = nodeWeapons.createChild();
		if nodeWeapon then
			DB.setValue(nodeWeapon, "isidentified", "number", nItemID);
			DB.setValue(nodeWeapon, "name", "string", sName);
			DB.setValue(nodeWeapon, "type", "number", 2);

			DB.setValue(nodeWeapon, "damagedice", "dice", aDamageDice);
			DB.setValue(nodeWeapon, "damagebonus", "number", nDamage);
			
			DB.setValue(nodeWeapon, "rangeincrement", "number", nRange);

			DB.setValue(nodeWeapon, "shortcut", "windowreference", "item", "....inventorylist." .. nodeItem.getName());
		end
	end
end

function initWeaponIDTracking()
	OptionsManager.registerCallback("MIID", onIDOptionChanged);
	DB.addHandler("charsheet.*.inventorylist.*.isidentified", "onUpdate", onItemIDChanged);
end

function onIDOptionChanged()
	for _,vChar in pairs(DB.getChildren("charsheet")) do
		for _,vWeapon in pairs(DB.getChildren(vChar, "weaponlist")) do
			checkWeaponIDChange(vWeapon);
		end
	end
end

function onItemIDChanged(nodeItemID)
	local nodeItem = nodeItemID.getChild("..");
	local nodeChar = nodeItemID.getChild("....");
	
	local sPath = nodeItem.getPath();
	for _,vWeapon in pairs(DB.getChildren(nodeChar, "weaponlist")) do
		local _,sRecord = DB.getValue(vWeapon, "shortcut", "", "");
		if sRecord == sPath then
			checkWeaponIDChange(vWeapon);
		end
	end
end

function checkWeaponIDChange(nodeWeapon)
	local _,sRecord = DB.getValue(nodeWeapon, "shortcut", "", "");
	if sRecord == "" then
		return;
	end
	local nodeItem = DB.findNode(sRecord);
	if not nodeItem then
		return;
	end
	
	local bItemID = LibraryData.getIDState("item", DB.findNode(sRecord), true);
	local bWeaponID = (DB.getValue(nodeWeapon, "isidentified", 1) == 1);
	if bItemID == bWeaponID then
		return;
	end
	
	local sName;
	if bItemID then
		sName = DB.getValue(nodeItem, "name", "");
	else
		sName = DB.getValue(nodeItem, "nonid_name", "");
		if sName == "" then
			sName = Interface.getString("item_unidentified");
		end
		sName = "** " .. sName .. " **";
	end
	DB.setValue(nodeWeapon, "name", "string", sName);
	
	if bItemID then
		DB.setValue(nodeWeapon, "isidentified", "number", 1);
	else
		DB.setValue(nodeWeapon, "isidentified", "number", 0);
	end
end

--
-- ACTIONS
--

function getBaseAttackRollStructures(nodeChar, sAttack)
	local rCreature = ActorManager.getActor("pc", nodeChar);

	local rAttack = {};
	rAttack.type = "attack";
	rAttack.label = sAttack;

	if string.match(string.lower(sAttack), "ranged") then
		rAttack.range = "R";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.ranged.total", 0);
		rAttack.stat = "dexterity";
	elseif string.match(string.lower(sAttack), "grapple") then
		rAttack.range = "M";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.grapple.total", 0);
		rAttack.stat = "strength";
	elseif string.match(string.lower(sAttack), "pummel") then
		rAttack.range = "M";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.pummel.total", 0);
		rAttack.stat = "strength";
	elseif string.match(string.lower(sAttack), "overbear") then
		rAttack.range = "M";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.overbear.total", 0);
		rAttack.stat = "strength";
	elseif string.match(string.lower(sAttack), "touch") then
		rAttack.range = "M";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.touch.total", 0);
	else
		rAttack.range = "M";
		rAttack.modifier = DB.getValue(nodeChar, "attackbonus.melee.total", 0);
		rAttack.stat = "strength";
	end
	
	return rCreature, rAttack;
end

function getWeaponAttackRollStructures(nodeWeapon, nAttack)
	if not nodeWeapon then
		return;
	end
	
	local nodeChar = nodeWeapon.getChild("...");
	local rActor = ActorManager.getActor("pc", nodeChar);

	local sName = DB.getValue(nodeWeapon, "name", "");
	
	local rAttack = {};
	rAttack.type = "attack";
	rAttack.label = sName;
	local nType = DB.getValue(nodeWeapon, "type", 0);
	if nType == 1 or nType == 2 then
		rAttack.range = "R";
		rAttack.stat = "dexterity";
	else
		rAttack.range = "M";
		rAttack.stat = "strength";
	end
	
	if sName:lower():match("touch") then
		rAttack.touch = true;
	end
	
	return rActor, rAttack;
end

function getWeaponDamageRollStructures(nodeWeapon)
	if not nodeWeapon then
		return;
	end
	
	local nodeChar = nodeWeapon.getChild("...");
	local rActor = ActorManager.getActor("pc", nodeChar);

	local nType = DB.getValue(nodeWeapon, "type", 0);
	local bRanged = (nType ~= 0);

	local rDamage = {};
	rDamage.type = "damage";
	rDamage.label = DB.getValue(nodeWeapon, "name", "");
	if bRanged then
		rDamage.range = "R";
	else
		rDamage.range = "M";
	end
	
	if nType ~= 1 then
		rDamage.stat = "strength";
	end
	
	rDamage.dice = DB.getValue(nodeWeapon, "damagedice", {});
	rDamage.modifier = DB.getValue(nodeWeapon, "damagetotalbonus", 0);
	
	return rActor, rDamage;
end

function rest(nodeChar)
	SpellManager.resetSpells(nodeChar);
	resetHealth(nodeChar);
end

function resetHealth(nodeChar)
	-- Clear temporary hit points and subdual damage
	DB.setValue(nodeChar, "hp.temporary", "number", 0);
	DB.setValue(nodeChar, "hp.nonlethal", "number", 0);
	
	-- Heal 1 hit point
	local nWounds = DB.getValue(nodeChar, "hp.wounds", 0);
	nWounds = nWounds - 1;
	if nWounds < 0 then
		nWounds = 0;
	end
	DB.setValue(nodeChar, "hp.wounds", "number", nWounds);
end
