-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

local parsed = false;
local rAttackRolls = {};
local rDamageRolls = {};

local hoverDamage = nil;
local hoverAttack = nil;
local clickDamage = nil;
local clickAttack = nil;

function onValueChanged()
	parsed = false;
end

function onHover(oncontrol)
	if dragging then
		return;
	end

	-- Reset selection when the cursor leaves the control
	if not oncontrol then
		-- Clear hover tracking
		hoverDamage = nil;
		hoverAttack = nil;
		
		-- Clear any selections
		setSelectionPosition(0);
	end
end

function onHoverUpdate(x, y)
	-- If we're typing or dragging, then exit
	if dragging then
		return;
	end

	-- Compute the locations of the relevant phrases, and the mouse
	local nMouseIndex = getIndexAt(x, y);
	if not parsed then
		parsed = true;
		rAttackRolls, rDamageRolls = CombatManager2.parseAttackLine(getActor(), getValue());
	end

	hoverDamage = nil;
	hoverAttack = nil;
	
	for i = 1, #rDamageRolls do
		if rDamageRolls[i].startpos < nMouseIndex and rDamageRolls[i].endpos > nMouseIndex then
			setCursorPosition(rDamageRolls[i].startpos);
			setSelectionPosition(rDamageRolls[i].endpos);

			hoverDamage = i;			
			setHoverCursor("hand");
			return;
		end
	end
	
	for i = 1, #rAttackRolls do
		if rAttackRolls[i].startpos < nMouseIndex and rAttackRolls[i].endpos > nMouseIndex then
			setCursorPosition(rAttackRolls[i].startpos);
			setSelectionPosition(rAttackRolls[i].endpos);

			hoverAttack = i;
			setHoverCursor("hand");
			return;
		end
	end

	-- Reset the cursor
	setHoverCursor("arrow");
end

function onClickDown(button, x, y)
	-- Suppress default processing to support dragging
	clickDamage = hoverDamage;
	clickAttack = hoverAttack;
	
	return true;
end

function onClickRelease(button, x, y)
	-- Enable edit mode on mouse release
	setFocus();
	
	local n = getIndexAt(x, y);
	
	setSelectionPosition(n);
	setCursorPosition(n);
	
	return true;
end

function getActor()
	return ActorManager.getActor("npc", window.getDatabaseNode());
end

function actionDamage(draginfo, rDamage)
	ActionDamage.performRoll(draginfo, getActor(), rDamage);
	return true;
end

function actionAttack(draginfo, rAttack)
	local rActor = getActor();
	local nAttack = ActorManager2.getBaseAttackBonus(rActor);
	
	local rRolls = {};
	for j = 1, rAttack.count do
		local rAttackAction = {};
		rAttackAction.label = rAttack.label;
		rAttackAction.order = j;
		rAttackAction.modifier = nAttack + (tonumber(rAttack.modifier) or 0);
		rAttackAction.touch = rAttack.touch;
		
		table.insert(rRolls, ActionAttack.getRoll(rActor, rAttackAction));
	end
	
	ActionsManager.performMultiAction(draginfo, rActor, "attack", rRolls);
	
	return true;
end

function onDoubleClick(x, y)
	if hoverDamage then
		return actionDamage(nil, rDamageRolls[hoverDamage]);
	end
	
	if hoverAttack then
		return actionAttack(nil, rAttackRolls[hoverAttack]);
	end
	
	return false;
end

function onDragStart(button, x, y, draginfo)
	if clickAttack or clickDamage then
		if clickDamage then
			actionDamage(draginfo, rDamageRolls[clickDamage]);
		end
		
		if clickAttack then
			actionAttack(draginfo, rAttackRolls[clickAttack]);
		end
		
		clickDamage = nil;
		clickAttack = nil;
		dragging = true;
	end
	
	return true;
end

function onDragEnd(dragdata)
	setCursorPosition(0);
	dragging = false;
end
