-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	-- Set the displays to what should be shown
	setTargetingVisible();
	setActiveVisible();
	setDefensiveVisible();
	setSpacingVisible();
	setEffectsVisible();

	-- Acquire token reference, if any
	linkToken();
	
	-- Set up the PC links
	onLinkChanged();
	
	-- Update the displays
	onFactionChanged();
	onHealthChanged();
	
	-- Register the deletion menu item for the host
	registerMenuItem(Interface.getString("list_menu_deleteitem"), "delete", 6);
	registerMenuItem(Interface.getString("list_menu_deleteconfirm"), "delete", 6, 7);
end

function updateDisplay()
	local sFaction = friendfoe.getStringValue();

	if DB.getValue(getDatabaseNode(), "active", 0) == 1 then
		name.setFont("sheetlabel");
		
		active_spacer_top.setVisible(true);
		active_spacer_bottom.setVisible(true);
		
		if sFaction == "friend" then
			setFrame("ctentrybox_friend_active");
		elseif sFaction == "neutral" then
			setFrame("ctentrybox_neutral_active");
		elseif sFaction == "foe" then
			setFrame("ctentrybox_foe_active");
		else
			setFrame("ctentrybox_active");
		end
	else
		name.setFont("sheettext");
		
		active_spacer_top.setVisible(false);
		active_spacer_bottom.setVisible(false);
		
		if sFaction == "friend" then
			setFrame("ctentrybox_friend");
		elseif sFaction == "neutral" then
			setFrame("ctentrybox_neutral");
		elseif sFaction == "foe" then
			setFrame("ctentrybox_foe");
		else
			setFrame("ctentrybox");
		end
	end
end

function linkToken()
	local imageinstance = token.populateFromImageNode(tokenrefnode.getValue(), tokenrefid.getValue());
	if imageinstance then
		TokenManager.linkToken(getDatabaseNode(), imageinstance);
	end
end

function onMenuSelection(selection, subselection)
	if selection == 6 and subselection == 7 then
		delete();
	end
end

function delete()
	local node = getDatabaseNode();
	if not node then
		close();
		return;
	end
	
	-- Remember node name
	local sNode = node.getNodeName();
	
	-- Clear any effects first, so that saves aren't triggered when initiative advanced
	effects.reset(false);

	-- Clear any NPC wounds first, so that stabilization rolls aren't triggered when initiative advanced
	local sClass, sRecord = link.getValue();
	if sClass ~= "charsheet" then
		wounds.setValue(0);
	end
	
	-- Move to the next actor, if this CT entry is active
	if DB.getValue(node, "active", 0) == 1 then
		CombatManager.nextActor();
	end

	-- Delete the database node and close the window
	node.delete();

	-- Update list information (global subsection toggles)
	windowlist.onVisibilityToggle();
	windowlist.onEntrySectionToggle();
end

function onLinkChanged()
	-- If a PC, then set up the links to the char sheet
	local sClass, sRecord = link.getValue();
	if sClass == "charsheet" then
		linkPCFields();
		name.setLine(false);
	end

	setActiveVisible();
end

function onMonsterChanged()
	setActiveVisible();
end

function onHealthChanged()
	local oHealth = ActorManager2.getHealth("ct", getDatabaseNode());
	local sColor = oHealth:woundColor();

	wounds.setColor(sColor);
	nonlethal.setColor(sColor);
	status.setValue(oHealth:status());
	
	local sClass,_ = link.getValue();
	if sClass ~= "charsheet" then
		idelete.setVisibility(oHealth:disabled());
	end
end

function onFactionChanged()
	-- Update the entry frame
	updateDisplay();

	-- If not a friend, then show visibility toggle
	if friendfoe.getStringValue() == "friend" then
		tokenvis.setVisible(false);
	else
		tokenvis.setVisible(true);
	end
end

function onVisibilityChanged()
	TokenManager.updateVisibility(getDatabaseNode());
	windowlist.onVisibilityToggle();
end

function onActiveChanged()
	setActiveVisible();
end

function linkPCFields()
	local nodeChar = link.getTargetDatabaseNode();
	if nodeChar then
		name.setLink(nodeChar.createChild("name", "string"), true);

		hp.setLink(nodeChar.createChild("hp.total", "number"));
		hptemp.setLink(nodeChar.createChild("hp.temporary", "number"));
		nonlethal.setLink(nodeChar.createChild("hp.nonlethal", "number"));
		wounds.setLink(nodeChar.createChild("hp.wounds", "number"));

		init.setLink(nodeChar.createChild("initiative.total", "number"), true);
		move.setLink(nodeChar.createChild("speed.total", "number"), true);
		bth.setLink(nodeChar.createChild("attackbonus.base", "number"), true);

		ac_final.setLink(nodeChar.createChild("ac.totals.general", "number"), true);
		ac_flatfooted.setLink(nodeChar.createChild("ac.totals.flatfooted", "number"), true);
		ac_touch.setLink(nodeChar.createChild("ac.totals.touch", "number"), true);
		
		sr.setLink(nodeChar.createChild("defenses.sr.total", "number"), true);
	end
end

--
-- SECTION VISIBILITY FUNCTIONS
--

function setTargetingVisible()
	local v = false;
	if activatetargeting.getValue() == 1 then
		v = true;
	end

	targetingicon.setVisible(v);
	
	sub_targeting.setVisible(v);
	
	frame_targeting.setVisible(v);

	target_summary.onTargetsChanged();
end

function setActiveVisible()
	local v = false;
	if activateactive.getValue() == 1 then
		v = true;
	end
	local sClass, sRecord = link.getValue();
	if sClass ~= "charsheet" and active.getValue() == 1 then
		v = true;
	end
	
	local bMonster;
	
	if sClass == "charsheet" then
		bMonster = false;
	else
		bMonster = (DB.getValue(getDatabaseNode(), "monster", "") ~= "false");
	end
	
	activeicon.setVisible(v);

	init.setVisible(v);
	init_label.setVisible(v);
	move.setVisible(v and not bMonster);
	monstermove.setVisible(v and bMonster);
	move_label.setVisible(v);
	bth.setVisible(v and not bMonster);
	monsterbth.setVisible(v and bMonster);
	bth_label.setVisible(v);
	atk.setVisible(v and not bMonster);
	monsteratk.setVisible(v and bMonster);
	atk_label.setVisible(v);
	sa.setVisible(v);
	sa_label.setVisible(v);
	
	frame_active.setVisible(v);
end

function setDefensiveVisible()
	local v = false;
	if activatedefensive.getValue() == 1 then
		v = true;
	end
	
	defensiveicon.setVisible(v);

	ac_final.setVisible(v);
	ac_label.setVisible(v);
	ac_flatfooted.setVisible(v);
	flatfootedac_label.setVisible(v);
	ac_touch.setVisible(v);
	touchac_label.setVisible(v);
	
	sr.setVisible(v);
	sr_label.setVisible(v);
	saves.setVisible(v);
	saves_label.setVisible(v);
	
	frame_defensive.setVisible(v);
end
	
function setSpacingVisible()
	local v = false;
	if activatespacing.getValue() == 1 then
		v = true;
	end

	spacingicon.setVisible(v);
	
	space.setVisible(v);
	spacelabel.setVisible(v);
	reach.setVisible(v);
	reachlabel.setVisible(v);
	
	frame_spacing.setVisible(v);
end

function setEffectsVisible()
	local v = false;
	if activateeffects.getValue() == 1 then
		v = true;
	end
	
	effecticon.setVisible(v);
	
	effects.setVisible(v);
	effects_iadd.setVisible(v);
	for _,w in pairs(effects.getWindows()) do
		w.idelete.setValue(0);
	end

	frame_effects.setVisible(v);

	effect_summary.onEffectsChanged();
end
