-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function getNPCRecordDisplayClass(vNode)
	local sRecordDisplayClass = "npc";
	if vNode then
		local sBasePath, sSecondPath = UtilityManager.getDataBaseNodePathSplit(vNode);
		if sBasePath == "reference" then
			if sSecondPath == "monsterdata" then
				sRecordDisplayClass = "monster";
			end
		end
	end
	return sRecordDisplayClass;
end

function getItemRecordDisplayClass(vNode)
	local sRecordDisplayClass = "item";
	if vNode then
		local sBasePath, sSecondPath = UtilityManager.getDataBaseNodePathSplit(vNode);
		if sBasePath == "reference" then
			if sSecondPath == "weapon" then
				sRecordDisplayClass = "referenceweapon";
			elseif sSecondPath == "armor" then
				sRecordDisplayClass = "referencearmor";
			else
				sRecordDisplayClass = "referenceequipment";
			end
		end
	end
	return sRecordDisplayClass;
end

function isItemIdentifiable(vNode)
	return (getItemRecordDisplayClass(vNode) == "item");
end

function getSpellSourceValue(vNode)
	return StringManager.split(DB.getValue(vNode, "level", ""), ",", true);
end

aRecords = {
	["npc"] = { 
		bExport = true,
		aDataMap = { "npc", "reference.npcdata", "reference.monsterdata" }, 
		aDisplayIcon = { "button_people", "button_people_down" },
		fRecordDisplayClass = getNPCRecordDisplayClass,
		-- sRecordDisplayClass = "npc", 
		aRecordDisplayClasses = { "npc", "monster" },
	},
	["item"] = { 
		bExport = true,
		bID = true,
		sIDOption = "MIID",
		fIsIdentifiable = isItemIdentifiable,
		aDataMap = { "item", "reference.equipment", "reference.weapon", "reference.armor", "reference.magicitems" }, 
		aDisplayIcon = { "button_items", "button_items_down" }, 
		sListDisplayClass = "masterindexitem_id",
		fRecordDisplayClass = getItemRecordDisplayClass,
		aRecordDisplayClasses = { "item", "referencearmor", "referenceweapon", "referenceequipment" },
		aCustomFilters = {
			["Type"] = { sField = "itemtype" },
		},
	},
	["class"] = {
		bExport = true,
		aDataMap = { "class", "reference.classes" }, 
		aDisplayIcon = { "button_classes", "button_classes_down" },
		sRecordDisplayClass = "referenceclass", 
	},
	["race"] = {
		bExport = true,
		aDataMap = { "race", "reference.races" }, 
		aDisplayIcon = { "button_races", "button_races_down" },
		sRecordDisplayClass = "referencerace", 
	},
	["spell"] = {
		bExport = true,
		aDataMap = { "spell", "reference.spells", "spelldesc" }, 
		aDisplayIcon = { "button_spells", "button_spells_down" },
		-- sRecordDisplayClass = "spell", 
		aCustomFilters = {
			["Source"] = { sField = "level", fGetValue = getSpellSourceValue };
		},
	},
};

aDefaultSidebarState = {
	["create"] = "charsheet,class,item,race,spell",
};

function onInit()
	for kDefSidebar,vDefSidebar in pairs(aDefaultSidebarState) do
		DesktopManager.setDefaultSidebarState(kDefSidebar, vDefSidebar);
	end
	for kRecordType,vRecordType in pairs(aRecords) do
		LibraryData.setRecordTypeInfo(kRecordType, vRecordType);
	end
end
