-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	ActionsManager.registerModHandler("ability", modRoll);
	ActionsManager.registerResultHandler("ability", onRoll);
end

function getRoll(rActor, sAbilityStat, bSecretRoll)
	local rRoll = {};
	rRoll.sType = "ability";
	rRoll.aDice = { "d20" };
	
	local nPrime = 0;
	local sActorType, nodeActor = ActorManager.getTypeAndNode(rActor);
	if nodeActor then
		if sActorType == "pc" then
			local nBonus = DB.getValue(nodeActor, "abilities." .. sAbilityStat .. ".bonus", 0);
			local nLevel = DB.getValue(nodeActor, "classes.slot1.level", 0);
			rRoll.nMod = nBonus + nLevel;
			nPrime = DB.getValue(nodeActor, "abilities." .. sAbilityStat .. ".prime", 0);
		else
			local nAbility = DB.getValue(nodeActor, sAbilityStat, 10);
			local nBonus = GameSystem.getAbilityBonus(nAbility);
			local nLevel = DB.getValue(nodeActor, "level", 0);
			rRoll.nMod = nBonus + nLevel;
			
			local sPrimes = DB.getValue(nodeActor, "primes", "");
			if string.match(sPrimes:lower(), sAbilityStat:sub(1, 3)) then
				nPrime = 1;
			end
		end
	end
	
	rRoll.sDesc = "[ABILITY]";
	rRoll.sDesc = rRoll.sDesc .. " " .. StringManager.capitalize(sAbilityStat);
	rRoll.sDesc = rRoll.sDesc .. " check";

	rRoll.bSecret = bSecretRoll;
	
	if nPrime == 2 then
		rRoll.sDesc = rRoll.sDesc .. " [SECONDARY]";
	elseif nPrime == 1 then
		rRoll.sDesc = rRoll.sDesc .. " [PRIME]";
	end

	return rRoll;
end

function performRoll(draginfo, rActor, sAbilityStat, bSecretRoll, bAddName)
	local rRoll = getRoll(rActor, sAbilityStat, bSecretRoll, bAddName);
	
	ActionsManager.performAction(draginfo, rActor, rRoll);
end

function getSkillRoll(rActor, vSkill)
	local rRoll = {};
	rRoll.sType = "ability";
	rRoll.aDice = { "d20" };
	
	local sAbility = DB.getValue(vSkill, "attribute", "");
	rRoll.nMod = DB.getValue(vSkill, "siegemodifier", 0);
	local bAddLevel = (DB.getValue(vSkill, "addlevel", 1) ~= 0);
	
	local nPrime = 0;
	local sActorType, nodeActor = ActorManager.getTypeAndNode(rActor);
	if nodeActor then
		if sActorType == "pc" then
			local nBonus = DB.getValue(nodeActor, "abilities." .. sAbility .. ".bonus", 0);
			rRoll.nMod = rRoll.nMod + nBonus;
			if bAddLevel then
				rRoll.nMod = rRoll.nMod + DB.getValue(nodeActor, "classes.slot1.level", 0);
			end
			nPrime = DB.getValue(nodeActor, "abilities." .. sAbility .. ".prime", 0);
		else
			local nAbility = DB.getValue(nodeActor, sAbility, 10);
			local nBonus = GameSystem.getAbilityBonus(nAbility);
			rRoll.nMod = rRoll.nMod + nBonus;
			if bAddLevel then
				rRoll.nMod = rRoll.nMod + DB.getValue(nodeActor, "level", 0);
			end
			
			local sPrimes = DB.getValue(nodeActor, "primes", "");
			if string.match(sPrimes:lower(), sAbility:sub(1, 3)) then
				nPrime = 1;
			end
		end
	end
	
	rRoll.sDesc = "[SKILL] " .. DB.getValue(vSkill, "value", "");

	if nPrime == 2 then
		rRoll.sDesc = rRoll.sDesc .. " [SECONDARY]";
	elseif nPrime == 1 then
		rRoll.sDesc = rRoll.sDesc .. " [PRIME]";
	end

	return rRoll;
end

function performSkillRoll(draginfo, rActor, vSkill)
	local rRoll = getSkillRoll(rActor, vSkill);
	
	ActionsManager.performAction(draginfo, rActor, rRoll);
end

function modRoll(rSource, rTarget, rRoll)
	local aAddDesc = {};
	local aAddDice = {};
	local nAddMod = 0;
	
	if rSource then
		local bEffects = false;

		local sActionStat = nil;
		local sAbility = string.match(rRoll.sDesc, "%[ABILITY%] (%w+) check");
		if sAbility then
			sAbility = string.lower(sAbility);
		end

		-- GET ACTION MODIFIERS
		local nEffectCount;
		aAddDice, nAddMod, nEffectCount = EffectManager.getEffectsBonus(rSource, {"ABIL"}, false, {sAbility});
		if (nEffectCount > 0) then
			bEffects = true;
		end
		
		-- GET STAT MODIFIERS
		local nBonusStat, nBonusEffects = ActorManager2.getAbilityEffectsBonus(rSource, sAbility);
		if nBonusEffects > 0 then
			bEffects = true;
			nAddMod = nAddMod + nBonusStat;
		end
		
		-- IF EFFECTS HAPPENED, THEN ADD NOTE
		if bEffects then
			local sEffects = "";
			local sMod = StringManager.convertDiceToString(aAddDice, nAddMod, true);
			if sMod ~= "" then
				sEffects = "[" .. Interface.getString("effects_tag") .. " " .. sMod .. "]";
			else
				sEffects = "[" .. Interface.getString("effects_tag") .. "]";
			end
			table.insert(aAddDesc, sEffects);
		end
	end
	
	if #aAddDesc > 0 then
		rRoll.sDesc = rRoll.sDesc .. " " .. table.concat(aAddDesc, " ");
	end
	for _,vDie in ipairs(aAddDice) do
		table.insert(rRoll.aDice, vDie);
	end
	rRoll.nMod = rRoll.nMod + nAddMod;
end

function onRoll(rSource, rTarget, rRoll)
	local rMessage = ActionsManager.createActionMessage(rSource, rRoll);
	
	local nFirstDie = 0;
	if #(rRoll.aDice) > 0 then
		nFirstDie = rRoll.aDice[1].result or 0;
	end
	if nFirstDie >= 20 then
		rMessage.text = rMessage.text .. " [CRITICAL SUCCESS]";
	elseif nFirstDie == 1 then
		rMessage.text = rMessage.text .. " [CRITICAL FAILURE]";
	end
	
	Comm.deliverChatMessage(rMessage);
end

