-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

OOB_MSGTYPE_APPLYDMG = "applydmg";

function onInit()
	OOBManager.registerOOBMsgHandler(OOB_MSGTYPE_APPLYDMG, handleApplyDamage);

	ActionsManager.registerModHandler("damage", modDamage);
	ActionsManager.registerResultHandler("damage", onDamage);
end

function handleApplyDamage(msgOOB)
	local rSource = ActorManager.getActor(msgOOB.sSourceType, msgOOB.sSourceNode);
	local rTarget = ActorManager.getActor(msgOOB.sTargetType, msgOOB.sTargetNode);
	if rTarget then
		rTarget.nOrder = msgOOB.nTargetOrder;
	end
	
	local nTotal = tonumber(msgOOB.nTotal) or 0;
	applyDamage(rSource, rTarget, (tonumber(msgOOB.nSecret) == 1), msgOOB.sRollType, msgOOB.sDamage, nTotal);
end

function notifyApplyDamage(rSource, rTarget, bSecret, sRollType, sDesc, nTotal)
	if not rTarget then
		return;
	end
	local sTargetType, sTargetNode = ActorManager.getTypeAndNodeName(rTarget);
	if sTargetType ~= "pc" and sTargetType ~= "ct" then
		return;
	end

	local msgOOB = {};
	msgOOB.type = OOB_MSGTYPE_APPLYDMG;
	
	if bSecret then
		msgOOB.nSecret = 1;
	else
		msgOOB.nSecret = 0;
	end
	msgOOB.sRollType = sRollType;
	msgOOB.nTotal = nTotal;
	msgOOB.sDamage = sDesc;
	msgOOB.sTargetType = sTargetType;
	msgOOB.sTargetNode = sTargetNode;
	msgOOB.nTargetOrder = rTarget.nOrder;

	local sSourceType, sSourceNode = ActorManager.getTypeAndNodeName(rSource);
	msgOOB.sSourceType = sSourceType;
	msgOOB.sSourceNode = sSourceNode;

	Comm.deliverOOBMessage(msgOOB, "");
end

function getRoll(rActor, rAction)
	local rRoll = {};
	rRoll.sType = "damage";
	rRoll.aDice = rAction.dice;
	rRoll.nMod = rAction.modifier or 0;
	
	rRoll.sDesc = "[DAMAGE";
	if rAction.order and rAction.order > 1 then
		rRoll.sDesc = rRoll.sDesc .. " #" .. rAction.order;
	end
	if rAction.range then
		rRoll.sDesc = rRoll.sDesc .. " (" .. rAction.range ..")";
	end
	rRoll.sDesc = rRoll.sDesc .. "] " .. rAction.label;
	
	if rAction.dmgtype and rAction.dmgtype ~= "" then
		rRoll.sDesc = rRoll.sDesc .. " [TYPE: " .. rAction.dmgtype .. "]";
	end
	
	return rRoll;
end

function performRoll(draginfo, rActor, rAction)
	local rRoll = getRoll(rActor, rAction);
	
	ActionsManager.performAction(draginfo, rActor, rRoll);
end

function modDamage(rSource, rTarget, rRoll)
	local aAddDesc = {};
	local aAddDice = {};
	local nAddMod = 0;
	
	-- Build attack type filter
	local aAttackFilter = {};
	local sAttackType = string.match(rRoll.sDesc, "%[DAMAGE%s*%(([^%)%]]+)%)%]");
	if sAttackType == "R" then
		table.insert(aAttackFilter, "ranged");
	elseif sAttackType == "M" then
		table.insert(aAttackFilter, "melee");
	end
	
	local aDamageTypes = decodeDamageTypes(false, rRoll.sDesc, rRoll.aDice, rRoll.nMod);
	rRoll.sDesc = string.gsub(rRoll.sDesc, " %[TYPE: ([^]]+)%]", "");
	
	if rSource then
		local bEffects = false;

		-- Get ability
		local sActionStat = nil;
		local sModStat = string.match(rRoll.sDesc, "%[MOD:(%w+)%]");
		if sModStat then
			sActionStat = GameSystem.ability_stol[sModStat];
		end
		if not sActionStat and sAttackType == "M" then
			sActionStat = "strength";
		end

		local aEffectDice = {};
		local nEffectMod = 0;

		-- Get ability modifiers
		if sActionStat then
			local nBonusStat, nBonusEffects = ActorManager2.getAbilityEffectsBonus(rSource, sActionStat);
			if nBonusEffects > 0 then
				bEffects = true;
				nAddMod = nAddMod + nBonusStat;
			end
		end
		
		-- Get damage modifiers
		local aEffects, nEffectCount = EffectManager.getEffectsBonusByType(rSource, "DMG", true, aAttackFilter, rTarget);
		if nEffectCount > 0 then
			bEffects = true;
			
			local sEffectBaseType = "";
			if #aDamageTypes > 0 then
				sEffectBaseType = aDamageTypes[1].sType or "";
			end
			
			for _,v in pairs(aEffects) do
				for _,v2 in ipairs(v.dice) do
					table.insert(aEffectDice, v2);
				end
				
				local nCurrentMod = v.mod;
				nEffectMod = nEffectMod + nCurrentMod;
				
				local aEffectDmgType = {};
				if sEffectBaseType ~= "" then
					table.insert(aEffectDmgType, sEffectBaseType);
				end
				for kWord, sWord in ipairs(v.remainder) do
					if StringManager.contains(GameSystem.dmgtypes, sWord) then
						table.insert(aEffectDmgType, sWord);
					end
				end
				if #aEffectDmgType > 0 then
					table.insert(aDamageTypes, { sType = table.concat(aEffectDmgType, ","), nDice = #(v.dice), nMod = nCurrentMod });
				else
					table.insert(aDamageTypes, { sType = "", nDice = #(v.dice), nMod = nCurrentMod });
				end
			end
		end
		
		-- IF EFFECTS HAPPENED, THEN ADD NOTE
		if bEffects then
			local sEffects = "";
			local sMod = StringManager.convertDiceToString(aEffectDice, nEffectMod, true);
			if sMod ~= "" then
				sEffects = "[" .. Interface.getString("effects_tag") .. " " .. sMod .. "]";
			else
				sEffects = "[" .. Interface.getString("effects_tag") .. "]";
			end
			table.insert(aAddDesc, sEffects);
			
			for _,vDie in ipairs(aEffectDice) do
				table.insert(aAddDice, vDie);
			end
			nAddMod = nAddMod + nEffectMod;
		end
	end
	
	if #aDamageTypes > 0 then
		table.insert(aAddDesc, encodeDamageTypes(aDamageTypes));
	end
	
	if #aAddDesc > 0 then
		rRoll.sDesc = rRoll.sDesc .. " " .. table.concat(aAddDesc, " ");
	end
	for _,vDie in ipairs(aAddDice) do
		table.insert(rRoll.aDice, vDie);
	end
	rRoll.nMod = rRoll.nMod + nAddMod;
end

function onDamage(rSource, rTarget, rRoll)
	local rMessage = ActionsManager.createActionMessage(rSource, rRoll);
	local nTotal = ActionsManager.total(rRoll);
	
	-- Handle minimum damage
	if string.match(rMessage.text, " %[MIN OVERRIDE%]") then
		rMessage.text = string.gsub(rMessage.text, " %[MIN OVERRIDE%]", "");
	elseif nTotal <= 0 and rRoll.aDice and #rRoll.aDice > 0 then
		local nMinDmgAdj = -(nTotal) + 1;
		rMessage.text = string.format("%s [MIN DMG ADJ %+d]", rMessage.text, nMinDmgAdj);
		rMessage.diemodifier = rMessage.diemodifier + nMinDmgAdj;
		nTotal = nTotal + nMinDmgAdj;
	end
	
	-- Send the chat message
	local bShowMsg = true;
	if rTarget and rTarget.nOrder and rTarget.nOrder ~= 1 then
		bShowMsg = false;
	end
	if bShowMsg then
		Comm.deliverChatMessage(rMessage);
	end

	-- Apply damage to the PC or CT entry referenced
	notifyApplyDamage(rSource, rTarget, rMessage.secret, rRoll.sType, rMessage.text, nTotal);
end

--
-- UTILITY FUNCTIONS
--

function encodeDamageTypes(aDamageTypes)
	local aOutputType = {};
	
	local nTypeCount = 0;
	local sLastDamageType = nil;
	local nLastDieCount = 0;
	local nLastMod = 0;
	local nLastTotal = nil;
	local nLastMult = 2;
	for _,v in ipairs(aDamageTypes) do
		local nCurrentMult = v.nMult or 2;
		
		local sCurrentDamageType = string.lower(v.sType);
		
		if sLastDamageType and ((sCurrentDamageType ~= sLastDamageType) or (nCurrentMult ~= nLastMult)) then
			local sDmgType;
			if sLastDamageType == "" then
				sDmgType = "[TYPE: untyped";
			else
				sDmgType = "[TYPE: " .. sLastDamageType;
			end
			sDmgType = sDmgType .. " (" .. nLastDieCount .. "D";
			if nLastMod > 0 then
				sDmgType = sDmgType .. "+" .. nLastMod;
			elseif nLastMod < 0 then
				sDmgType = sDmgType .. nLastMod;
			end
			if nLastTotal then
				sDmgType = sDmgType .. "=" .. nLastTotal;
			end
			sDmgType = sDmgType .. ")";
			if nLastMult ~= 2 then
				sDmgType = sDmgType .. "(x" .. nLastMult .. ")";
			end
			sDmgType = sDmgType .. "]";
			
			table.insert(aOutputType, sDmgType);
			
			nTypeCount = nTypeCount + 1;
			nLastDieCount = 0;
			nLastMod = 0;
		end
		
		sLastDamageType = sCurrentDamageType;
		nLastDieCount = nLastDieCount + v.nDice;
		nLastMod = nLastMod + v.nMod;
		nLastMult = nCurrentMult;
		nLastTotal = v.nTotal;
	end
	if sLastDamageType and ((sLastDamageType ~= "") or (nLastMult ~= 2)) then
		local sDmgType;
		if sLastDamageType == "" then
			sDmgType = "[TYPE: untyped";
		else
			sDmgType = "[TYPE: " .. sLastDamageType;
		end
		if nTypeCount > 0 then
			sDmgType = sDmgType .. " (" .. nLastDieCount .. "D";
			if nLastMod > 0 then
				sDmgType = sDmgType .. "+" .. nLastMod;
			elseif nLastMod < 0 then
				sDmgType = sDmgType .. nLastMod;
			end
			if nLastTotal then
				sDmgType = sDmgType .. "=" .. nLastTotal;
			end
			sDmgType = sDmgType .. ")";
		end
		if nLastMult ~= 2 then
			sDmgType = sDmgType .. "(x" .. nLastMult .. ")";
		end
		sDmgType = sDmgType .. "]";
		
		table.insert(aOutputType, sDmgType);
	end
	
	return table.concat(aOutputType, " ");
end

function decodeDamageTypes(bRolled, sDesc, aDice, nMod)
	local aDamageTypes = {};
	
	local nDamageDice = 0;
	local nDamageMod = 0;
	
	for sDamageClause in string.gmatch(sDesc, "%[TYPE: ([^]]+)%]") do
		local sDamageType = string.match(sDamageClause, "^([,%w%s]+)");
		if sDamageType then
			sDamageType = StringManager.trim(sDamageType);
			
			local nTotal = nil;
			local sDieCount, sSign, sMod = string.match(sDamageClause, "%((%d+)D([%+%-]?)(%d*)");
			local nDieCount = tonumber(sDieCount) or 0;
			local nDieMod = tonumber(sMod) or 0;
			if nDieCount > 0 or nDieMod > 0 then
				if sSign == "-" then
					nDieMod = 0 - nDieMod;
				end
			else
				nDieCount = #aDice - nDamageDice;
				nDieMod = nMod - nDamageMod;
			end

			nDamageDice = nDamageDice + nDieCount;
			nDamageMod = nDamageMod + nDieMod;

			if sDamageType == "untyped" then
				sDamageType = "";
			end
			
			local rDamageType = { sType = sDamageType, nDice = nDieCount, nMod = nDieMod };

			local sMult = string.match(sDamageClause, "%(x(%d+)%)");
			if sMult then
				rDamageType.nMult = tonumber(sMult) or 2;
			end
			local sTotal = string.match(sDamageClause, "%(%d+D[%+%-]?%d*=(%d+)%)");
			if sTotal then
				rDamageType.nTotal = tonumber(sTotal) or 0;
			end
			
			table.insert(aDamageTypes, rDamageType);
		end
	end
	
	if (nDamageDice < #aDice) or (nDamageMod ~= nMod) then
		local rDamageType = { sType = "", nDice = (#aDice - nDamageDice), nMod = (nMod - nDamageMod) };
		if bRolled then
			rDamageType.nTotal = 0;
			if (nDamageDice < #aDice) then
				for i = nDamageDice + 1, #aDice do
					rDamageType.nTotal = rDamageType.nTotal + aDice[i].result;
				end
			end
			rDamageType.nTotal = rDamageType.nTotal + rDamageType.nMod;
		end
		
		table.insert(aDamageTypes, rDamageType);
	end
	
	return aDamageTypes;
end

function getDamageTypesFromString(sDamageTypes)
	local sLower = string.lower(sDamageTypes);
	local aSplit = StringManager.split(sLower, ",", true);
	
	local aDamageTypes = {};
	for k, v in ipairs(aSplit) do
		if StringManager.contains(GameSystem.dmgtypes, v) then
			table.insert(aDamageTypes, v);
		end
	end
	
	return aDamageTypes;
end

--
-- DAMAGE APPLICATION
--

function getReductionType(rSource, rTarget, sEffectType)
	local aEffects = EffectManager.getEffectsByType(rTarget, sEffectType, {}, rSource);
	
	local aFinal = {};
	for _,v in pairs(aEffects) do
		local rReduction = {};
		
		rReduction.mod = v.mod;
		rReduction.aNegatives = {};
		for _,vType in pairs(v.remainder) do
			if #vType > 1 and ((vType:sub(1,1) == "!") or (vType:sub(1,1) == "~")) then
				if StringManager.contains(GameSystem.dmgtypes, vType:sub(2)) then
					table.insert(rReduction.aNegatives, vType:sub(2));
				end
			end
		end

		for _,vType in pairs(v.remainder) do
			if vType ~= "untyped" and vType ~= "" and vType:sub(1,1) ~= "!" and vType:sub(1,1) ~= "~" then
				if StringManager.contains(GameSystem.dmgtypes, vType) or vType == "all" then
					aFinal[vType] = rReduction;
				end
			end
		end
	end
	
	return aFinal;
end

function checkReductionTypeHelper(rMatch, aDmgType)
	if not rMatch or (rMatch.mod ~= 0) then
		return false;
	end
	if #(rMatch.aNegatives) > 0 then
		local bMatchNegative = false;
		for _,vNeg in pairs(rMatch.aNegatives) do
			if StringManager.contains(aDmgType, vNeg) then
				bMatchNegative = true;
				break;
			end
		end
		return not bMatchNegative;
	end
	return true;
end

function checkReductionType(aReduction, aDmgType)
	for _,sDmgType in pairs(aDmgType) do
		if checkReductionTypeHelper(aReduction[sDmgType], aDmgType) or checkReductionTypeHelper(aReduction["all"], aDmgType) then
			return true;
		end
	end
	
	return false;
end

function checkNumericalReductionTypeHelper(rMatch, aDmgType, nLimit)
	if not rMatch or (rMatch.mod == 0) then
		return 0;
	end

	local bMatch = false;
	if #rMatch.aNegatives > 0 then
		local bMatchNegative = false;
		for _,vNeg in pairs(rMatch.aNegatives) do
			if StringManager.contains(aDmgType, vNeg) then
				bMatchNegative = true;
				break;
			end
		end
		if not bMatchNegative then
			bMatch = true;
		end
	else
		bMatch = true;
	end
	
	local nAdjust = 0;
	if bMatch then
		nAdjust = rMatch.mod - (rMatch.nApplied or 0);
		if nLimit then
			nAdjust = math.min(nAdjust, nLimit);
		end
		rMatch.nApplied = (rMatch.nApplied or 0) + nAdjust;
	end
	
	return nAdjust;
end

function checkNumericalReductionType(aReduction, aDmgType, nLimit)
	local nAdjust = 0;
	
	for _,sDmgType in pairs(aDmgType) do
		if nLimit then
			local nSpecificAdjust = checkNumericalReductionTypeHelper(aReduction[sDmgType], aDmgType, nLimit);
			nAdjust = nAdjust + nSpecificAdjust;
			local nGlobalAdjust = checkNumericalReductionTypeHelper(aReduction["all"], aDmgType, nLimit - nSpecificAdjust);
			nAdjust = nAdjust + nGlobalAdjust;
		else
			nAdjust = nAdjust + checkNumericalReductionTypeHelper(aReduction[sDmgType], aDmgType);
			nAdjust = nAdjust + checkNumericalReductionTypeHelper(aReduction["all"], aDmgType);
		end
	end
	
	return nAdjust;
end

function getDamageAdjust(rSource, rTarget, nDamage, rDamageOutput)
	local nDamageAdjust = 0;
	local nNonlethal = 0;
	local bVulnerable = false;
	local bResist = false;
	
	-- Get target damage adjustment effects
	local aImmune = getReductionType(rSource, rTarget, "IMMUNE");
	local aVuln = getReductionType(rSource, rTarget, "VULN");
	local aResist = getReductionType(rSource, rTarget, "RESIST");
	
	-- Handle immune all
	if aImmune["all"] then
		nDamageAdjust = 0 - nDamage;
		bResist = true;
		return nDamageAdjust, bVulnerable, bResist;
	end
	
	-- Iterate through each damage type entry
	local nVulnApplied = 0;
	for k, v in pairs(rDamageOutput.aDamageTypes) do
		-- Get individual damage types for each damage clause
		local bNonlethal = false;
		local aSrcDmgClauseTypes = {};
		local aTemp = StringManager.split(k, ",", true);
		for _,vType in ipairs(aTemp) do
			if vType ~= "untyped" and vType ~= "" then
				table.insert(aSrcDmgClauseTypes, vType);
				if vType == "nonlethal" then
					bNonlethal = true;
				end
			end
		end

		-- Handle standard immunity, vulnerability and resistance
		local bLocalVulnerable = checkReductionType(aVuln, aSrcDmgClauseTypes);
		local bLocalResist = checkReductionType(aResist, aSrcDmgClauseTypes);
		local bLocalImmune = checkReductionType(aImmune, aSrcDmgClauseTypes);
		
		-- Calculate adjustment
		-- Vulnerability = +50%
		-- Resistance = half
		-- Immunity = none
		local nLocalDamageAdjust = 0;
		if bLocalImmune then
			nLocalDamageAdjust = -v;
			bResist = true;
		else
			-- Handle standard vulnerability
			if bLocalVulnerable then
				nLocalDamageAdjust = math.floor(v / 2);
				bVulnerable = true;
			end
			-- Handle numerical vulnerability
			local nLocalVulnerable = checkNumericalReductionType(aVuln, aSrcDmgClauseTypes);
			if nLocalVulnerable ~= 0 then
				nLocalDamageAdjust = nLocalDamageAdjust + nLocalVulnerable;
				bVulnerable = true;
			end
			-- Handle standard resistance
			if bLocalResist then
				local nAdj = math.ceil((nLocalDamageAdjust + v) / 2);
				nLocalDamageAdjust = nLocalDamageAdjust - nAdj;
				bResist = true;
			end
			-- Handle numerical resistance
			local nLocalResist = checkNumericalReductionType(aResist, aSrcDmgClauseTypes, nLocalDamageAdjust + v);
			if nLocalResist ~= 0 then
				nLocalDamageAdjust = nLocalDamageAdjust - nLocalResist;
				bResist = true;
			end
		end
		
		-- Calculate nonlethal damage
		local nNonlethalAdjust = 0;
		if bNonlethal and ((v + nLocalDamageAdjust) > 0) then
			nNonlethalAdjust = v + nLocalDamageAdjust;
		end

		-- Apply damage adjustment
		nDamageAdjust = nDamageAdjust + nLocalDamageAdjust - nNonlethalAdjust;
		nNonlethal = nNonlethal + nNonlethalAdjust;
	end

	return nDamageAdjust, nNonlethal, bVulnerable, bResist;
end

function decodeDamageText(nDamage, sDamageDesc)
	local rDamageOutput = {};
	rDamageOutput.sType = "damage";
	rDamageOutput.sTypeOutput = "Damage";
	rDamageOutput.sVal = "" .. nDamage;
	rDamageOutput.nVal = nDamage;
	
	if string.match(sDamageDesc, "%[HEAL") then
		if string.match(sDamageDesc, "%[TEMP%]") then
			rDamageOutput.sType = "nTempHP";
			rDamageOutput.sTypeOutput = "Temporary hit points";
		else
			rDamageOutput.sType = "heal";
			rDamageOutput.sTypeOutput = "Heal";
		end
	elseif string.match(sDamageDesc, "%[REGEN") then
		rDamageOutput.sType = "regen";
		rDamageOutput.sTypeOutput = "Regeneration";
	elseif nDamage < 0 then
		rDamageOutput.sType = "heal";
		rDamageOutput.sTypeOutput = "Heal";
		rDamageOutput.sVal = "" .. (0 - nDamage);
		rDamageOutput.nVal = 0 - nDamage;
	else
		-- Determine range
		rDamageOutput.sRange = string.match(sDamageDesc, "%[DAMAGE %((%w)%)%]") or "";
		rDamageOutput.aDamageFilter = {};
		if rDamageOutput.sRange == "M" then
			table.insert(rDamageOutput.aDamageFilter, "melee");
		elseif rDamageOutput.sRange == "R" then
			table.insert(rDamageOutput.aDamageFilter, "ranged");
		end

		-- Determine energy types
		rDamageOutput.aDamageTypes = {};
		local nDamageRemaining = nDamage;
		for sDamageType in string.gmatch(sDamageDesc, "%[TYPE: ([^%]]+)%]") do
			local sDmgType = StringManager.trim(string.match(sDamageType, "^([^(%]]+)"));
			local sDice, sTotal = string.match(sDamageType, "%(([%d%+%-D]+)%=(%d+)%)");
			local nDmgTypeTotal = tonumber(sTotal) or nDamageRemaining;

			if rDamageOutput.aDamageTypes[sDmgType] then
				rDamageOutput.aDamageTypes[sDmgType] = rDamageOutput.aDamageTypes[sDmgType] + nDmgTypeTotal;
			else
				rDamageOutput.aDamageTypes[sDmgType] = nDmgTypeTotal;
			end
			if not rDamageOutput.sFirstDamageType then
				rDamageOutput.sFirstDamageType = sDmgType;
			end

			nDamageRemaining = nDamageRemaining - nDmgTypeTotal;
			if nDamageRemaining <= 0 then
				break;
			end
		end
		if nDamageRemaining > 0 then
			rDamageOutput.aDamageTypes[""] = nDamageRemaining;
		end
	end
	
	return rDamageOutput;
end

function applyDamage(rSource, rTarget, bSecret, sRollType, sDamage, nTotal)
	local nTotalHP = 0;
	local nTempHP = 0;
	local nNonLethal = 0;
	local nWounds = 0;

	local aNotifications = {};
	
	-- Get actor health
	local sTargetType, nodeTarget = ActorManager.getTypeAndNode(rTarget);
	if sTargetType ~= "pc" and sTargetType ~= "ct" then
		return;
	end
	
	if sTargetType == "pc" then
		nTotalHP = DB.getValue(nodeTarget, "hp.total", 0);
		nTempHP = DB.getValue(nodeTarget, "hp.temporary", 0);
		nNonlethal = DB.getValue(nodeTarget, "hp.nonlethal", 0);
		nWounds = DB.getValue(nodeTarget, "hp.wounds", 0);
	else
		nTotalHP = DB.getValue(nodeTarget, "hp", 0);
		nTempHP = DB.getValue(nodeTarget, "hptemp", 0);
		nNonlethal = DB.getValue(nodeTarget, "nonlethal", 0);
		nWounds = DB.getValue(nodeTarget, "wounds", 0);
	end
	
	-- Get current health status
	local oOldHealth = ActorManager2.getHealth(sTargetType, nodeTarget);

	-- Decode damage text
	local rDamageOutput = decodeDamageText(nTotal, sDamage);
	
	-- Healing
	if rDamageOutput.sType == "heal" or rDamageOutput.sType == "regen" then
		if nWounds <= 0 and nNonlethal <= 0 then
			if rDamageOutput.sType == "heal" then
				table.insert(aNotifications, "[NOT WOUNDED]");
			end
		else
			-- Calculate heal amounts
			local nHealAmount = rDamageOutput.nVal;
			
			local nNonlethalHealAmount = math.min(nHealAmount, nNonlethal);
			nNonlethal = nNonlethal - nNonlethalHealAmount;

			local nOriginalWounds = nWounds;
			
			local nWoundHealAmount = math.min(nHealAmount, nWounds);
			nWounds = nWounds - nWoundHealAmount;
			
			rDamageOutput.nVal = nNonlethalHealAmount + nWoundHealAmount;
			if nWoundHealAmount > 0 then
				rDamageOutput.sVal = "" .. nWoundHealAmount;
				if nNonlethalHealAmount > 0 then
					rDamageOutput.sVal = rDamageOutput.sVal .. " (+" .. nNonlethalHealAmount .. " NL)";
				end
			elseif nNonlethalHealAmount > 0 then
				rDamageOutput.sVal = "" .. nNonlethalHealAmount .. " NL";
			else
				rDamageOutput.sVal = "0";
			end
		end

	-- Temporary hit points
	elseif rDamageOutput.sType == "nTempHP" then
		nTempHP = math.max(nTempHP, nTotal);

	-- Damage
	else
		if rSource and rTarget and rTarget.nOrder then
			local aTargetedDamage = EffectManager.getEffectsBonusByType(rSource, {"DMG"}, true, rDamageOutput.aDamageFilter, rTarget, true);

			local nDamageEffectTotal = 0;
			local nDamageEffectCount = 0;
			for k, v in pairs(aTargetedDamage) do
				local nSubTotal = StringManager.evalDice(v.dice, v.mod);
				
				local sDamageType = rDamageOutput.sFirstDamageType;
				if sDamageType then
					sDamageType = sDamageType .. "," .. k;
				else
					sDamageType = k;
				end

				rDamageOutput.aDamageTypes[sDamageType] = (rDamageOutput.aDamageTypes[sDamageType] or 0) + nSubTotal;
				
				nDamageEffectTotal = nDamageEffectTotal + nSubTotal;
				nDamageEffectCount = nDamageEffectCount + 1;
			end
			nTotal = nTotal + nDamageEffectTotal;

			if nDamageEffectCount > 0 then
				if nDamageEffectTotal ~= 0 then
					local sFormat = "[" .. Interface.getString("effects_tag") .. " %+d]";
					table.insert(aNotifications, string.format(sFormat, nDamageEffectTotal));
				else
					table.insert(aNotifications, "[" .. Interface.getString("effects_tag") .. "]");
				end
			end
		end
		
		-- Apply damage adjustment effects
		local nDamageAdjust, nNonlethalDmgAmount, bVulnerable, bResist = getDamageAdjust(rSource, rTarget, nTotal, rDamageOutput);
		local nAdjustedDamage = nTotal + nDamageAdjust;
		if nAdjustedDamage < 0 then
			nAdjustedDamage = 0;
		end
		if bResist then
			if nAdjustedDamage <= 0 then
				table.insert(aNotifications, "[RESISTED]");
			else
				table.insert(aNotifications, "[PARTIALLY RESISTED]");
			end
		end
		if bVulnerable then
			table.insert(aNotifications, "[VULNERABLE]");
		end
		
		-- Apply damage to temporary hit points first
		if nTempHP > 0 and nAdjustedDamage > 0 then
			if nAdjustedDamage > nTempHP then
				nAdjustedDamage = nAdjustedDamage - nTempHP;
				nTempHP = 0;
				table.insert(aNotifications, "[PARTIALLY ABSORBED]");
			else
				nTempHP = nTempHP - nAdjustedDamage;
				nAdjustedDamage = 0;
				table.insert(aNotifications, "[ABSORBED]");
			end
		end

		-- Update the damage output variable to reflect adjustments
		rDamageOutput.nVal = nAdjustedDamage;
		if nAdjustedDamage > 0 then
			rDamageOutput.sVal = "" .. nAdjustedDamage;
			if nNonlethalDmgAmount > 0 then
				rDamageOutput.sVal = rDamageOutput.sVal .. " (+" .. nNonlethalDmgAmount .. " NL)";
			end
		elseif nNonlethalDmgAmount > 0 then
			rDamageOutput.sVal = "" .. nNonlethalDmgAmount .. " NL";
		else
			rDamageOutput.sVal = "0";
		end

		-- Apply remainder to wounds
		local nOriginalWounds = nWounds;
		nWounds = nWounds + nAdjustedDamage;
		if nWounds < 0 then
			nWounds = 0;
		end
		local nOriginalNonlethal = nNonlethal;
		nNonlethal = nNonlethal + nNonlethalDmgAmount;
		if nNonlethal < 0 then
			nNonlethal = 0;
		end

	end

	-- Set new health
	if sTargetType == "pc" then
		DB.setValue(nodeTarget, "hp.temporary", "number", nTempHP);
		DB.setValue(nodeTarget, "hp.wounds", "number", nWounds);
		DB.setValue(nodeTarget, "hp.nonlethal", "number", nNonlethal);
	else
		DB.setValue(nodeTarget, "hptemp", "number", nTempHP);
		DB.setValue(nodeTarget, "wounds", "number", nWounds);
		DB.setValue(nodeTarget, "nonlethal", "number", nNonlethal);
	end

	-- Check for status change
	local bShowStatus = false;
	if ActorManager.getFaction(rTarget) == "friend" then
		bShowStatus = not OptionsManager.isOption("SHPC", "off");
	else
		bShowStatus = not OptionsManager.isOption("SHNPC", "off");
	end
	if bShowStatus then
		local oNewHealth = ActorManager2.getHealth(sTargetType, nodeTarget);
		if oOldHealth:status() ~= oNewHealth:status() then
			table.insert(aNotifications, "[" .. Interface.getString("combat_tag_status") .. ": " .. oNewHealth:status() .. "]");
		end
	end

	-- Output
	messageDamage(rSource, rTarget, bSecret, rDamageOutput.sTypeOutput, sDamage, rDamageOutput.sVal, table.concat(aNotifications, " "));
end

function messageDamage(rSource, rTarget, bSecret, sDamageType, sDamageDesc, sTotal, sExtraResult)
	if not (rTarget or sExtraResult ~= "") then
		return;
	end
	
	local msgShort = {font = "msgfont"};
	local msgLong = {font = "msgfont"};

	if sDamageType == "Heal" or sDamageType == "Temporary hit points" then
		msgShort.icon = "roll_heal";
		msgLong.icon = "roll_heal";
	else
		msgShort.icon = "roll_damage";
		msgLong.icon = "roll_damage";
	end

	msgShort.text = sDamageType .. " ->";
	msgLong.text = sDamageType .. " [" .. sTotal .. "] ->";
	if rTarget then
		msgShort.text = msgShort.text .. " [to " .. rTarget.sName .. "]";
		msgLong.text = msgLong.text .. " [to " .. rTarget.sName .. "]";
	end
	
	if sExtraResult and sExtraResult ~= "" then
		msgLong.text = msgLong.text .. " " .. sExtraResult;
	end
	
	ActionsManager.messageResult(bSecret, rSource, rTarget, msgLong, msgShort);
end
