-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	CombatManager.setCustomSort(CombatManager.sortfuncStandard);
	CombatManager.setCustomDrop(onDrop);

	CombatManager.setCustomAddNPC(addNPC);
	CombatManager.setCustomNPCSpaceReach(getNPCSpaceReach);

	CombatManager.setCustomRoundStart(onRoundStart);
	CombatManager.setCustomCombatReset(resetInit);
end

--
-- PER ROUND INITIATIVE
--

function onRoundStart(nRound)
	if OptionsManager.isOption("HRIR", "on") then
		rollInit();
	end
end

--
-- DROP HANDLING
--

function onDrop(rSource, rTarget, draginfo)
	local sDragType = draginfo.getType();

	-- Effect targeting
	if sDragType == "effect_targeting" then
		if User.isHost() then
			onEffectTargetingDrop(rSource, rTarget, draginfo);
			return true;
		end
	end
end

function onEffectTargetingDrop(rSource, rTarget, draginfo)
	local sTargetCT = ActorManager.getCTNodeName(rTarget);
	if sTargetCT ~= "" then
		local sRefClass, sEffectNode = draginfo.getShortcutData();
		if sRefClass and sEffectNode then
			if sRefClass == "ct_effect" then
				EffectManager.addEffectTarget(sEffectNode, sTargetCT);
			end
		end
	end
end

--
-- ADD FUNCTIONS
--

function getNPCSpaceReach(nodeNPC)
	local nSpace = GameSystem.GameSystem.getDistanceUnitsPerGrid();
	local nReach = nSpace;
	
	local sSize = DB.getValue(nodeNPC, "size", "");
	if sSize:lower() == "large" then
		nSpace = nSpace * 2;
		nReach = nReach * 2;
	end

	return nSpace, nReach;
end

function addNPC(sClass, nodeNPC, sName)
	local nodeEntry, nodeLastMatch = CombatManager.addNPCHelper(nodeNPC, sName);

	local bMonster = true;
	if sClass == "monster" then
		bMonster = true;
		CampaignDataManager2.convertMonsterToNPC(nodeNPC, nodeEntry);
	else
		bMonster = (DB.getValue(nodeEntry, "monster", "") ~= "false");
	end
	
	-- HD
	local sHD;
	if bMonster then
		sHD = DB.getValue(nodeEntry, "monsterhd", "");
	else
		sHD = DB.getValue(nodeEntry, "hd", "");
	end
	local nHD = tonumber(sHD:match("%d+")) or 0;
	
	-- HP
	local nHP;
	if bMonster then
		nHP = DB.getValue(nodeEntry, "monsterhp", 0);
	else
		nHP = DB.getValue(nodeEntry, "hp", 0);
	end
	if nHP == 0 then
		local sCleanHD = string.gsub(string.lower(sHD),"[^%d%-d%+]",""); 
		
		local sOptHRNH = OptionsManager.getOption("HRNH");
		if sOptHRNH == "max" then
			nHP = StringManager.evalDiceString(sCleanHD, true, true);
		elseif sOptHRNH == "random" then
			nHP = math.max(StringManager.evalDiceString(sCleanHD, true), 1);
		else
			-- Calculate average
			local aDice, nMod = StringManager.convertStringToDice(sCleanHD);
			nHP = nMod;
			for _,v in ipairs(aDice) do
				nHP = nHP + ((tonumber(v:sub(2)) or 0) + 1) / 2;
			end
			nHP = math.max(math.floor(nHP), 0);
		end
	end
	DB.setValue(nodeEntry, "hp", "number", nHP);

	-- Offsensive properties
	local nBTH;
	if bMonster then
		nBTH = DB.getValue(nodeEntry, "monsterbth", 0);
	else
		nBTH = DB.getValue(nodeEntry, "bth", 0);
	end
	if nBTH == 0 then
		nBTH = nHD;
	end
	DB.setValue(nodeEntry, "bth", "number", nBTH);
	DB.setValue(nodeEntry, "monsterbth", "number", nBTH);
	
	-- Defensive properties
	local nAC;
	if bMonster then
		local sAC = DB.getValue(nodeEntry, "monsterac", "10")
		nAC = tonumber(string.match(sAC, "^%s*(%d+)")) or 10;
	else
		nAC = DB.getValue(nodeEntry, "ac", 10);
	end
	DB.setValue(nodeEntry, "ac_final", "number", nAC);
	DB.setValue(nodeEntry, "ac_flatfooted", "number", nAC);
	DB.setValue(nodeEntry, "ac_touch", "number", 10);
	
	-- Track additional damage types and intrinsic effects
	local aEffects = {};
	
	-- Decode special qualities
	local sSpecialQualities = DB.getValue(nodeEntry, "sa", "");
	
	local aSQWords = StringManager.parseWords(sSpecialQualities:lower());
	local i = 1;
	while aSQWords[i] do
		-- Spell Resistance
		if StringManager.isWord(aSQWords[i], "sr") then
			if StringManager.isNumberString(aSQWords[i+1]) then
				i = i + 1;
				DB.setValue(nodeEntry, "sr", "number", tonumber(aSQWords[i]) or 0);
			end
		
		-- Regeneration
		elseif StringManager.isWord(aSQWords[i], "regeneration") then
			if StringManager.isNumberString(aSQWords[i+1]) then
				i = i + 1;
				table.insert(aEffects, "REGEN: " .. aSQWords[i]);
			end
		
		-- Resistance
		elseif StringManager.isWord(aSQWords[i], "resistance") then
			if StringManager.isWord(aSQWords[i-1], GameSystem.dmgtypes) then
				table.insert(aEffects, "RESIST: " .. aSQWords[i-1]);
			else
				if StringManager.isWord(aSQWords[i+1], "to") then
					i = i + 1;
				end
	
				while aSQWords[i+1] do
					if StringManager.isWord(aSQWords[i+1], "and") then
						-- SKIP
					elseif StringManager.isWord(aSQWords[i+1], GameSystem.dmgtypes) then
						i = i + 1;
						table.insert(aEffects, "RESIST: " .. aSQWords[i]);
					else
						break;
					end

					i = i + 1;
				end
			end
		
		-- Vulnerability
		elseif StringManager.isWord(aSQWords[i], "vulnerability") and StringManager.isWord(aSQWords[i+1], "to") then
			if StringManager.isWord(aSQWords[i-1], GameSystem.dmgtypes) then
				table.insert(aEffects, "VULN: " .. aSQWords[i-1]);
			else
				if StringManager.isWord(aSQWords[i+1], "to") then
					i = i + 1;
				end
			
				while aSQWords[i+1] do
					if StringManager.isWord(aSQWords[i+1], "and") then
						-- SKIP
					elseif StringManager.isWord(aSQWords[i+1], GameSystem.dmgtypes) then
						if StringManager.isWord(aSQWords[i+2], "protection") then
							break;
						end
						table.insert(aEffects, "VULN: " .. aSQWords[i+1]);
					else
						break;
					end

					i = i + 1;
				end
			end

		-- Immunity
		elseif StringManager.isWord(aSQWords[i], "immunity") and StringManager.isWord(aSQWords[i+1], "to") then
			if StringManager.isWord(aSQWords[i-1], GameSystem.energytypes) then
				table.insert(aEffects, "IMMUNE: " .. aSQWords[i-1]);
			else
				if StringManager.isWord(aSQWords[i+1], "to") then
					i = i + 1;
				end
			
				while aSQWords[i+1] do
					if StringManager.isWord(aSQWords[i+1], {"and", "half", "full"}) then
						-- SKIP
					elseif StringManager.isWord(aSQWords[i+1], GameSystem.energytypes) then
						if StringManager.isWord(aSQWords[i+2], "half") then
							table.insert(aEffects, "RESIST: " .. aSQWords[i+1]);
						else
							table.insert(aEffects, "IMMUNE: " .. aSQWords[i+1]);
						end
					else
						break;
					end

					i = i + 1;
				end
			end
		end

		-- ITERATE SPECIAL QUALITIES DECODE
		i = i + 1;
	end

	-- ADD DECODED EFFECTS
	if #aEffects > 0 then
		EffectManager.addEffect("", "", nodeEntry, { sName = table.concat(aEffects, "; "), nDuration = 0, nGMOnly = 1 }, false);
	end

	-- Roll initiative and sort
	local nInitDieSides = 10;
	local sOptHRID = OptionsManager.getOption("HRID");
	if sOptHRID == "100" then
		nInitDieSides = 100;
	elseif sOptHRID == "20" then
		nInitDieSides = 20;
	end
	local sOptHRIN = OptionsManager.getOption("HRIN");
	if sOptHRIN == "group" then
		if nodeLastMatch then
			local nLastInit = DB.getValue(nodeLastMatch, "initresult", 0);
			DB.setValue(nodeEntry, "initresult", "number", nLastInit);
		else
			DB.setValue(nodeEntry, "initresult", "number", math.random(nInitDieSides));
		end
	else
		DB.setValue(nodeEntry, "initresult", "number", math.random(nInitDieSides));
	end

	return nodeEntry;
end

--
-- RESET FUNCTIONS
--

function resetInit()
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		DB.setValue(vChild, "initresult", "number", 0);
	end
end

function resetEffects()
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local nodeEffects = vChild.getChild("effects");
		if nodeEffects then
			for _, vEffect in pairs(nodeEffects.getChildren()) do
				vEffect.delete();
			end
		end
	end
end

function clearExpiringEffects(bShort)
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local nodeEffects = vChild.getChild("effects");
		if nodeEffects then
			for _, vEffect in pairs(nodeEffects.getChildren()) do
				local sLabel = DB.getValue(vEffect, "label", "");
				local nDuration = DB.getValue(vEffect, "duration", 0);
				
				if nDuration ~= 0 or sLabel == "" then
					if bShort then
						if nDuration > 50 then
							DB.setValue(vEffect, "duration", "number", nDuration - 50);
						else
							vEffect.delete();
						end
					else
						vEffect.delete();
					end
				end
			end
		end
	end
end

function rest(bShort)
	CombatManager.resetInit();
	clearExpiringEffects(bShort);
	
	if not bShort then
		for _,vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
			local sClass, sRecord = DB.getValue(vChild, "link", "", "");
			if sClass == "charsheet" and sRecord ~= "" then
				local nodePC = DB.findNode(sRecord);
				if nodePC then
					CharManager.rest(nodePC);
				end
			end
		end
	end
end

function rollEntryInit(nodeEntry)
	if not nodeEntry then
		return;
	end
	
	local nInitDieSides = 10;
	local sOptHRID = OptionsManager.getOption("HRID");
	if sOptHRID == "100" then
		nInitDieSides = 100;
	elseif sOptHRID == "20" then
		nInitDieSides = 20;
	end

	-- For PCs, we always roll unique initiative
	local sClass, sRecord = DB.getValue(nodeEntry, "link", "", "");
	if sClass == "charsheet" then
		local nInitResult = math.random(nInitDieSides);
		local rActor = ActorManager.getActor("pc", sRecord);
		if rActor then
			nInitResult = nInitResult + DB.getValue(ActorManager.getCreatureNode(rActor), "initiative.total", 0);
		end
		
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end
	
	-- For NPCs, if NPC init option is not group, then roll unique initiative
	local sOptHRIN = OptionsManager.getOption("HRIN");
	if sOptHRIN ~= "group" then
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end

	-- For NPCs with group option enabled
	
	-- Get the entry's database node name and creature name
	local sStripName = CombatManager.stripCreatureNumber(DB.getValue(nodeEntry, "name", ""));
	if sStripName == "" then
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end
		
	-- Iterate through list looking for other creature's with same name
	local nLastInit = nil;
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		if vChild.getName() ~= nodeEntry.getName() then
			local sTemp = CombatManager.stripCreatureNumber(DB.getValue(vChild, "name", ""));
			if sTemp == sStripName then
				local nChildInit = DB.getValue(vChild, "initresult", 0);
				if nChildInit ~= -10000 then
					nLastInit = nChildInit;
				end
			end
		end
	end
	
	-- If we found similar creatures, then match the initiative of the last one found
	if nLastInit then
		DB.setValue(nodeEntry, "initresult", "number", nLastInit);
	else
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
	end
end

function rollInit(sType)
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local bRoll = true;
		if sType then
			local sClass,_ = DB.getValue(vChild, "link", "", "");
			if sType == "npc" and sClass == "charsheet" then
				bRoll = false;
			elseif sType == "pc" and sClass ~= "charsheet" then
				bRoll = false;
			end
		end
		
		if bRoll then
			DB.setValue(vChild, "initresult", "number", -10000);
		end
	end

	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local bRoll = true;
		if sType then
			local sClass,_ = DB.getValue(vChild, "link", "", "");
			if sType == "npc" and sClass == "charsheet" then
				bRoll = false;
			elseif sType == "pc" and sClass ~= "charsheet" then
				bRoll = false;
			end
		end
		
		if bRoll then
			rollEntryInit(vChild);
		end
	end
end

--
-- PARSE CT ATTACK LINE
--

function parseAttackLine(rActor, sText)
	local rAttackRolls = {};
	local rDamageRolls = {};

	sText = sText:gsub("", "-");

	local aClauses = {};
	local nIndex = 1;
	local nStart, nEnd, nStart2, nEnd2, nStart3, nEnd3;
	local sPhrase;
	while nIndex < #sText do
		nStart, nEnd = string.find(sText, "%s+or%s+", nIndex);
		nStart2, nEnd2 = string.find(sText, "%s*;%s*", nIndex);
		nStart3, nEnd3 = string.find(sText, "%)%s*,%s*", nIndex);
		nStart4, nEnd4 = string.find(sText, "%s+and%s+", nIndex);
		
		if nStart2 and (not nStart or nStart > nStart2) then
			nStart = nStart2;
			nEnd = nEnd2;
		end
		if nStart3 and (not nStart or nStart > nStart3) then
			nStart = nStart3 + 1;
			nEnd = nEnd3;
		end
		if nStart4 and (not nStart or nStart > nStart4) then
			nStart = nStart4;
			nEnd = nEnd4;
		end
		
		if nStart then
			sPhrase = string.sub(sText, nIndex, nEnd - 1);
		else
			sPhrase = string.sub(sText, nIndex);
		end
		
		nPhraseStart, sAll, sAttackCount, sAttackLabel, nDamageStart, sDamage, nDamageEnd, nPhraseEnd = string.match(sPhrase, '()((%d*) ?([%w%s%[%]%+%-]*) %(()([^%)]*)()%))()');
		if nPhraseStart then
			local rAttack = {};
			rAttack.startpos = nIndex + nPhraseStart - 1;
			rAttack.endpos = nIndex + nPhraseEnd - 1;
			
			local rDamage = {};
			rDamage.startpos = nIndex + nDamageStart - 1;
			rDamage.endpos = nIndex + nDamageEnd - 1;
			
			-- Clean up the attack count field (i.e. magical weapon bonuses up front, no attack count)
			local nAttackCount = 1;
			if #sAttackCount then
				nAttackCount = tonumber(sAttackCount) or 1;
				if nAttackCount < 1 then
					nAttackCount = 1;
				end
			end

			-- Watch for touch attacks
			local aLabelWords = StringManager.parseWords(sAttackLabel:lower());
			if StringManager.contains(aLabelWords, "touch") then
				rAttack.touch = true;
			end
			
			-- Decode any attack bonus
			local nBonusStart, sAttackModifier = string.match(sAttackLabel, "()%s+([%+%-]?%d+)$");
			if nBonusStart then
				sAttackLabel = sAttackLabel:sub(1, nBonusStart);
			end
			
			-- Capitalize first letter of label
			sAttackLabel = StringManager.capitalize(sAttackLabel);
			
			rAttack.label = sAttackLabel;
			rAttack.count = nAttackCount;
			rAttack.modifier = sAttackModifier;
			
			rDamage.label = sAttackLabel;
			local aDamageDice, nDamageMod = StringManager.convertStringToDice(sDamage)
			rDamage.dice = aDamageDice;
			rDamage.modifier = nDamageMod;
			
			-- Add to roll list
			table.insert(rAttackRolls, rAttack);
			table.insert(rDamageRolls, rDamage);
		end
		
		if nStart then
			nIndex = nEnd + 1;
		else
			nIndex = #sText;
		end
	end
	
	return rAttackRolls, rDamageRolls;
end
