-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

-- Effects that adjust rolls
-- REGEN: # 					Regeneration
-- DMGO: # [<type>]				Ongoing damage
-- ATK: # [melee|ranged]		Attack bonus (targeted)
-- DMG: # [melee|ranged]		Damage bonus (targeted)
-- HEAL: #						Heal bonus
-- AC: #						AC bonus (targeted)
-- STR|DEX|CON|INT|WIS|CHA: #	Ability score bonus
-- CONC1|CONC2|CONC3|CONC4		Concealment (targeted)
-- COVER1|COVER2|COVER3|COVER4	Cover (targeted)
-- Blinded						(Attacker: Applies CONC4 for target)(Defender: AC: -5) (targeted)
-- Invisible					CONC4 (targeted)
-- Prone						AC: -5
-- Stunned						AC: -2
-- Cowering						AC: -2

OOB_MSGTYPE_APPLYEFF = "applyeff";
OOB_MSGTYPE_EXPIREEFF = "expireeff";

local nLocked = 0;

function onInit()
	OOBManager.registerOOBMsgHandler(OOB_MSGTYPE_APPLYEFF, handleApplyEffect);
	OOBManager.registerOOBMsgHandler(OOB_MSGTYPE_EXPIREEFF, handleExpireEffect);

	CombatManager.setCustomInitChange(processEffects);
end

function handleApplyEffect(msgOOB)
	-- Get the combat tracker node
	local nodeCTEntry = DB.findNode(msgOOB.sTargetNode);
	if not nodeCTEntry then
		ChatManager.SystemMessage(Interface.getString("ct_error_effectapplyfail") .. " (" .. msgOOB.sTargetNode .. ")");
		return;
	end
	
	-- Reconstitute the effect details
	local rEffect = {};
	rEffect.sName = msgOOB.sName;
	rEffect.nDuration = tonumber(msgOOB.nDuration) or 1;
	rEffect.nInit = tonumber(msgOOB.nInit) or 0;
	rEffect.sSource = msgOOB.sSource;
	rEffect.nGMOnly = tonumber(msgOOB.nGMOnly) or 0;
	
	local sEffectTargetNode = msgOOB.sEffectTargetNode or "";
	
	-- Apply the damage
	addEffect(msgOOB.user, msgOOB.identity, nodeCTEntry, rEffect, true, sEffectTargetNode);
end

function handleExpireEffect(msgOOB)
	-- Get the effect and combat tracker node
	local nodeEffect = DB.findNode(msgOOB.sEffectNode);
	if not nodeEffect then
		ChatManager.SystemMessage(Interface.getString("ct_error_effectdeletefail") .. " (" .. msgOOB.sEffectNode .. ")");
		return;
	end
	local nodeActor = nodeEffect.getChild("...");
	if not nodeActor then
		ChatManager.SystemMessage(Interface.getString("ct_error_effectmissingactor") .. " (" .. msgOOB.sEffectNode .. ")");
		return;
	end
	
	-- Get the parameters
	local nExpireType = tonumber(msgOOB.nExpireType) or 0;
	
	-- Apply the damage
	expireEffect(nodeActor, nodeEffect, nExpireType);
end

function notifyApply(rEffect, vTargets, sEffectTargetNode)
	local msgOOB = {};
	msgOOB.type = OOB_MSGTYPE_APPLYEFF;
	if User.isHost() then
		msgOOB.user = "";
	else
		msgOOB.user = User.getUsername();
	end
	msgOOB.identity = User.getIdentityLabel();

	msgOOB.sName = rEffect.sName or "";
	msgOOB.nDuration = rEffect.nDuration or 1;
	msgOOB.nInit = rEffect.nInit or 0;
	msgOOB.sSource = rEffect.sSource or ""; 
	msgOOB.nGMOnly = rEffect.nGMOnly or 0; 

	msgOOB.sEffectTargetNode = sEffectTargetNode;
	
	if type(vTargets) == "table" then
		for _, v in pairs(vTargets) do
			msgOOB.sTargetNode = v;
			Comm.deliverOOBMessage(msgOOB, "");
		end
	else
		msgOOB.sTargetNode = vTargets;
		Comm.deliverOOBMessage(msgOOB, "");
	end
end

function notifyExpire(varEffect, nMatch)
	if type(varEffect) == "databasenode" then
		varEffect = varEffect.getNodeName();
	elseif type(varEffect) ~= "string" then
		return;
	end
	
	local msgOOB = {};
	msgOOB.type = OOB_MSGTYPE_EXPIREEFF;
	msgOOB.sEffectNode = varEffect;
	msgOOB.nExpireType = nMatch;
	
	Comm.deliverOOBMessage(msgOOB, "");
end

function setEffect(nodeEffect, rEffect)
	DB.setValue(nodeEffect, "label", "string", rEffect.sName);
	DB.setValue(nodeEffect, "isgmonly", "number", rEffect.nGMOnly);
	DB.setValue(nodeEffect, "duration", "number", rEffect.nDuration);
	if nodeEffect.getChild("source_name") then
		DB.setValue(nodeEffect, "source_name", "string", rEffect.sSource);
	end
	if nodeEffect.getChild("init") then
		DB.setValue(nodeEffect, "init", "number", rEffect.nInit);
	end
end

function getEffect(nodeEffect)
	return { 
			sName = DB.getValue(nodeEffect, "label", ""), 
			nGMOnly = DB.getValue(nodeEffect, "isgmonly", 0),
			nDuration = DB.getValue(nodeEffect, "duration", 0),
			nSource = DB.getValue(nodeEffect, "source_name", ""),
			nInit = DB.getValue(nodeEffect, "init", 0),
			};
end

--
-- EFFECTS
--

function message(sMsg, nodeCT, gmflag, sUser)
	local msg = {font = "msgfont", icon = "roll_effect", text = sMsg};

	if nodeCT then
		msg.text = msg.text .. " [on " .. DB.getValue(nodeCT, "name", "") .. "]";
	end
	
	if sUser then
		if sUser == "" then
			Comm.addChatMessage(msg);
		else
			Comm.deliverChatMessage(msg, sUser);
		end
	elseif gmflag then
		msg.secret = true;
		if User.isHost() then
			Comm.addChatMessage(msg);
		else
			Comm.deliverChatMessage(msg, User.getUsername());
		end
	else
		Comm.deliverChatMessage(msg);
	end
end

function parseEffect(s)
	local aEffects = {};
	
	local sEffectClause;
	for sEffectClause in string.gmatch(s, "([^;]*);?") do
		local aWords = StringManager.parseWords(sEffectClause, "%[%]%(%):");
		if #aWords > 0 then
			local sType = string.match(aWords[1], "^([^:]+):");
			local aDice = {};
			local nMod = 0;
			local aRemainder = {};
			local nRemainderIndex = 1;
			if sType then
				nRemainderIndex = 2;

				local sValueCheck = "";
				local sTypeRemainder = string.sub(aWords[1], #sType + 2);
				if sTypeRemainder == "" then
					sValueCheck = aWords[2] or "";
					nRemainderIndex = nRemainderIndex + 1;
				else
					sValueCheck = sTypeRemainder;
				end
				
				if StringManager.isDiceString(sValueCheck) then
					aDice, nMod = StringManager.convertStringToDice(sValueCheck);
				elseif sValueCheck ~= "" then
					table.insert(aRemainder, sValueCheck);
				end
			end
			
			for i = nRemainderIndex, #aWords do
				table.insert(aRemainder, aWords[i]);
			end

			table.insert(aEffects, {type = sType or "", mod = nMod, dice = aDice, 
					remainder = aRemainder, original = StringManager.trim(sEffectClause)});
		end
	end
	
	return aEffects;
end

function rebuildParsedEffect(aEffectComps)
	local aEffect = {};
	
	for kComp, rComp in ipairs(aEffectComps) do
		local aComp = {};

		if rComp.type ~= "" then
			table.insert(aComp, rComp.type .. ":");
		end

		local sDiceString = StringManager.convertDiceToString(rComp.dice, rComp.mod);
		if sDiceString ~= "" then
			table.insert(aComp, sDiceString);
		end
		
		for kRemainder, sRemainder in ipairs(rComp.remainder) do
			table.insert(aComp, sRemainder);
		end
		
		table.insert(aEffect, table.concat(aComp, " "));
	end
	
	return table.concat(aEffect, "; ");
end

function getEffectsString(nodeCTEntry, bPublicOnly)
	local aOutputEffects = {};
	
	-- Iterate through each effect
	local aSorted = {};
	for _,nodeChild in pairs(DB.getChildren(nodeCTEntry, "effects")) do
		table.insert(aSorted, nodeChild);
	end
	table.sort(aSorted, function (a, b) return a.getName() < b.getName() end);
	for _,v in pairs(aSorted) do
		if DB.getValue(v, "isactive", 0) == 1 then
			local sLabel = DB.getValue(v, "label", "");

			local bAddEffect = true;
			local bGMOnly = false;
			if sLabel == "" then
				bAddEffect = false;
			elseif DB.getValue(v, "isgmonly", 0) == 1 then
				if User.isHost() and not bPublicOnly then
					bGMOnly = true;
				else
					bAddEffect = false;
				end
			end

			if bAddEffect then
				local aAddCompList = {};
				local bTargeted = false;
				local aEffectComps = EffectManager.parseEffect(sLabel);
				for _,vComp in ipairs(aEffectComps) do
					if vComp.remainder[1] == "TRGT" then
						bTargeted = true;
					else
						table.insert(aAddCompList, vComp);
					end
				end
				
				if isTargetedEffect(v) then
					local sTargets = table.concat(getEffectTargets(v, true), ",");
					table.insert(aAddCompList, 1, {type = "", mod = 0, dice = {}, remainder = {"[TRGT: " .. sTargets .. "]"}});
				elseif bTargeted then
					table.insert(aAddCompList, 1, {type = "", mod = 0, dice = {}, remainder = {"[TRGT]"}});
				end

				local nDuration = DB.getValue(v, "duration", 0);
				if nDuration > 0 then
					table.insert(aAddCompList,  {type = "", mod = 0, dice = {}, remainder = {"[D: " .. nDuration .. "]"}});
				end
				
				local sOutputLabel = rebuildParsedEffect(aAddCompList);
				if bGMOnly then
					sOutputLabel = "(" .. sOutputLabel .. ")";
				end

				table.insert(aOutputEffects, sOutputLabel);
			end
		end
	end
	
	-- Return the final effect list string
	return table.concat(aOutputEffects, " | ");
end

function isGMEffect(nodeActor, nodeEffect)
	local bGMOnly = false;
	if nodeEffect then
		bGMOnly = (DB.getValue(nodeEffect, "isgmonly", 0) == 1);
	end
	if nodeActor then
		if (DB.getValue(nodeActor, "friendfoe", "") ~= "friend") and 
				(DB.getValue(nodeActor, "tokenvis", 0) == 0) then
			bGMOnly = true;
		end
	end
	return bGMOnly;
end

function isTargetedEffect(nodeEffect)
	return (DB.getChildCount(nodeEffect, "targets") > 0);
end

function getEffectTargets(nodeEffect, bUseName)
	local aTargets = {};
	
	for _,nodeTarget in pairs(DB.getChildren(nodeEffect, "targets")) do
		local sNode = DB.getValue(nodeTarget, "noderef", "");
		if bUseName then
			local nodeTargetCT = DB.findNode(sNode);
			table.insert(aTargets, DB.getValue(nodeTargetCT, "name", ""));
		else
			table.insert(aTargets, sNode);
		end
	end

	return aTargets;
end

function removeEffect(nodeCTEntry, sEffPatternToRemove)
	-- VALIDATE
	if not sEffPatternToRemove then
		return;
	end

	-- COMPARE EFFECT NAMES TO EFFECT TO REMOVE
	for _,nodeEffect in pairs(DB.getChildren(nodeCTEntry, "effects")) do
		local s = DB.getValue(nodeEffect, "label", "");
		if string.match(s, sEffPatternToRemove) then
			nodeEffect.delete();
			return;
		end
	end
end

function addEffect(sUser, sIdentity, nodeCT, rNewEffect, bShowMsg, sEffectTargetNode)
	if not nodeCT or not rNewEffect or not rNewEffect.sName then
		return;
	end
	rNewEffect.nDuration = rNewEffect.nDuration or 1;
	rNewEffect.nInit = rNewEffect.nInit or 0;
	
	-- Get actor effects list
	local sCTName = DB.getValue(nodeCT, "name", "");
	local nodeEffectsList = nodeCT.createChild("effects");
	if not nodeEffectsList then
		return;
	end
	
	-- Parse new effect
	local aNewEffectParse = parseEffect(rNewEffect.sName);
	
	-- Ignore effects that are duplicate
	for k, v in pairs(nodeEffectsList.getChildren()) do
		if (DB.getValue(v, "label", "") == rNewEffect.sName) and 
					(DB.getValue(v, "init", 0) == rNewEffect.nInit) and
					(DB.getValue(v, "duration", 0) == rNewEffect.nDuration) then
			local sMsg = "Effect ['" .. rNewEffect.sName .. "'] ";
			if rNewEffect.nDuration > 0 then
				sMsg = sMsg .. " [D:" .. rNewEffect.nDuration .. "] ";
			end
			sMsg = sMsg .. "-> [ALREADY EXISTS]";
			message(sMsg, nodeCT, false, sUser);
			return;
		end
	end
	
	-- Write effect details
	local nodeTargetEffect = nodeEffectsList.createChild();
	DB.setValue(nodeTargetEffect, "label", "string", rNewEffect.sName);
	DB.setValue(nodeTargetEffect, "duration", "number", rNewEffect.nDuration);
	DB.setValue(nodeTargetEffect, "init", "number", rNewEffect.nInit);
	DB.setValue(nodeTargetEffect, "isgmonly", "number", rNewEffect.nGMOnly);
	DB.setValue(nodeTargetEffect, "isactive", "number", 1);

	-- Handle effect targeting data
	if sEffectTargetNode and sEffectTargetNode ~= "" then
		addEffectTarget(nodeTargetEffect, sEffectTargetNode);
	end
	
	-- Build output message
	local msg = {font = "msgfont", icon = "roll_effect"};
	msg.text = "Effect ['" .. rNewEffect.sName .. "'] ";
	if rNewEffect.nDuration > 0 then
		msg.text = msg.text .. " [D:" .. rNewEffect.nDuration .. "] ";
	end
	msg.text = msg.text .. "-> [to " .. sCTName .. "]";
	if rNewEffect.sSource and rNewEffect.sSource ~= "" then
		DB.setValue(nodeTargetEffect, "source_name", "string", rNewEffect.sSource);
		msg.text = msg.text .. " [by " .. DB.getValue(DB.findNode(rNewEffect.sSource), "name", "") .. "]";
	end
	
	-- Send Message
	if bShowMsg then
		if isGMEffect(nodeCT, nodeTargetEffect) then
			if sUser == "" then
				msg.secret = true;
				Comm.addChatMessage(msg);
			elseif sUser ~= "" then
				Comm.addChatMessage(msg);
				Comm.deliverChatMessage(msg, sUser);
			end
		else
			Comm.deliverChatMessage(msg);
		end
	end
end

function expireEffect(nodeActor, nodeEffect, nExpireComponent, bOverride)
	if not nodeEffect then
		return false;
	end

	-- Parse effect
	local sEffect = DB.getValue(nodeEffect, "label", "");

	-- Determine visibility
	local bGMOnly = isGMEffect(nodeActor, nodeEffect);

	-- Delete
	sMsg = "Effect ['" .. sEffect .. "'] -> [EXPIRED]";
	nodeEffect.delete();

	-- Notification
	message(sMsg, nodeActor, bGMOnly);
	return true;
end

function applyOngoingDamageAdjustment(nodeActor, nodeEffect, rEffectComp)
	-- Exit if no dice or modifier
	if #(rEffectComp.dice) == 0 and rEffectComp.mod == 0 then
		return;
	end
	
	-- Build message
	local aResults = {};
	if rEffectComp.type == "REGEN" then
		-- Make sure affected actor has wounds
		if DB.getValue(nodeActor, "wounds", 0) == 0 then
			return;
		end
		
		table.insert(aResults, "[REGEN] Regeneration");

	else
		table.insert(aResults, "[DAMAGE] Ongoing Damage");

		if #(rEffectComp.remainder) > 0 then
			table.insert(aResults, "[TYPE: " .. string.lower(table.concat(rEffectComp.remainder, ",")) .. "]");
		end
	end

	-- Apply results
	local rTarget = ActorManager.getActorFromCT(nodeActor);
	local rRoll = { sType = "damage", sDesc = table.concat(aResults, " "), aDice = rEffectComp.dice, nMod = rEffectComp.mod };
	if isGMEffect(nodeActor, nodeEffect) then
		rRoll.bSecret = true;
	end
	ActionsManager.roll(nil, rTarget, rRoll);
end

function processEffects(nodeCurrentActor, nodeNewActor)
	-- Setup current and new initiative values
	local nCurrentInit = 10000;
	if nodeCurrentActor then
		nCurrentInit = DB.getValue(nodeCurrentActor, "initresult", 0); 
	end
	local nNewInit = -10000;
	if nodeNewActor then
		nNewInit = DB.getValue(nodeNewActor, "initresult", 0);
	end
	
	-- Iterate through each actor
	for _,nodeActor in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		-- Iterate through each effect
		for _,nodeEffect in pairs(DB.getChildren(nodeActor, "effects")) do
			-- Make sure effect is active
			local nActive = DB.getValue(nodeEffect, "isactive", 0);
			if (nActive ~= 0) then
				-- Handle start of turn effects (ongoing damage and regeneration)
				if nodeActor == nodeNewActor then
					local sEffName = DB.getValue(nodeEffect, "label", "");
					local listEffectComp = parseEffect(sEffName);
					for _,rEffectComp in ipairs(listEffectComp) do
						if rEffectComp.type == "DMGO" or rEffectComp.type == "REGEN" then
							applyOngoingDamageAdjustment(nodeActor, nodeEffect, rEffectComp);
						end
					end
				end

				-- Check for effect expiration
				local nDuration = DB.getValue(nodeEffect, "duration", 0);
				if (nDuration > 0) then
					local nEffInit = DB.getValue(nodeEffect, "init", 0);
					if nEffInit < nCurrentInit and nEffInit >= nNewInit then
						nDuration = nDuration - 1;
						
						if (nDuration <= 0) then
							expireEffect(nodeActor, nodeEffect, 0);
						else
							DB.setValue(nodeEffect, "duration", "number", nDuration);
						end
					end
				end
			end
		end
	end
end

function getEffectsByType(rActor, sEffectType, aFilter, rFilterActor, bTargetedOnly)
	if not rActor then
		return {};
	end
	
	local results = {};
	
	-- SEPARATE FILTERS
	local aRangeFilter = {};
	local aOtherFilter = {};
	if aFilter then
		for _,v in pairs(aFilter) do
			if type(v) ~= "string" then
				table.insert(aOtherFilter, v);
			elseif StringManager.contains(GameSystem.rangetypes, v) then
				table.insert(aRangeFilter, v);
			else
				table.insert(aOtherFilter, v);
			end
		end
	end
	
	-- DETERMINE WHETHER EFFECT COMPONENT WE ARE LOOKING FOR SUPPORTS TARGETING
	local bTargetSupport = StringManager.isWord(sEffectType, GameSystem.targetableeffectcomps);
	
	-- ITERATE THROUGH EFFECTS
	for _,v in pairs(DB.getChildren(ActorManager.getCTNode(rActor), "effects")) do
		-- MAKE SURE EFFECT IS ACTIVE
		local nActive = DB.getValue(v, "isactive", 0);
		if (nActive ~= 0) then
			-- PARSE EFFECT
			local sLabel = DB.getValue(v, "label", "");
			local effect_list = EffectManager.parseEffect(sLabel);

			-- IF COMPONENT WE ARE LOOKING FOR SUPPORTS TARGETS, THEN CHECK AGAINST OUR TARGET
			local bTargeted = isTargetedEffect(v);
			if not bTargeted or isEffectTarget(v, rFilterActor) then

				-- LOOK THROUGH EFFECT CLAUSES FOR A TYPE (or TYPE/SUBTYPE) MATCH
				local nMatch = 0;
				for kEffectComp, rEffectComp in ipairs(effect_list) do
					-- STRIP OUT ENERGY OR BONUS TYPES FOR SUBTYPE COMPARISON
					local aEffectRangeFilter = {};
					local aEffectOtherFilter = {};
					local j = 1;
					while rEffectComp.remainder[j] do
						if rEffectComp.remainder[j] == "cold" and rEffectComp.remainder[j + 1] and rEffectComp.remainder[j + 1] == "iron" then
							j = j + 1;
						elseif StringManager.contains(GameSystem.dmgtypes, rEffectComp.remainder[j]) then
							-- SKIP
						elseif StringManager.contains(GameSystem.rangetypes, rEffectComp.remainder[j]) then
							table.insert(aEffectRangeFilter, rEffectComp.remainder[j]);
						else
							table.insert(aEffectOtherFilter, rEffectComp.remainder[j]);
						end
						
						j = j + 1;
					end
				
					-- CHECK TO MAKE SURE THIS COMPONENT MATCHES THE ONE WE'RE SEARCHING FOR
					local comp_match = false;
					if rEffectComp.type == sEffectType then

						-- CHECK FOR EFFECT CHECKS FOR TARGETS ONLY (EITHER BY EFFECT OR CONDITION)
						if bTargetedOnly and not bTargeted then
							comp_match = false;
						else
							comp_match = true;
						end
					
						-- CHECK THE FILTERS
						if #aEffectRangeFilter > 0 then
							local bRangeMatch = false;
							for _,v2 in pairs(aRangeFilter) do
								if StringManager.contains(aEffectRangeFilter, v2) then
									bRangeMatch = true;
									break;
								end
							end
							if not bRangeMatch then
								comp_match = false;
							end
						end
						if #aEffectOtherFilter > 0 then
							local bOtherMatch = false;
							for _,v2 in pairs(aOtherFilter) do
								if type(v2) == "table" then
									local bOtherTableMatch = true;
									for k3, v3 in pairs(v2) do
										if not StringManager.contains(aEffectOtherFilter, v3) then
											bOtherTableMatch = false;
											break;
										end
									end
									if bOtherTableMatch then
										bOtherMatch = true;
										break;
									end
								elseif StringManager.contains(aEffectOtherFilter, v2) then
									bOtherMatch = true;
									break;
								end
							end
							if not bOtherMatch then
								comp_match = false;
							end
						end
					end

					-- WE FOUND A MATCH
					if comp_match then
						nMatch = kEffectComp;
						if nActive == 1 then
							table.insert(results, rEffectComp);
						end
					end
				end -- END EFFECT COMPONENT LOOP
			end -- END TARGET CHECK
		end  -- END ACTIVE CHECK
	end  -- END EFFECT LOOP
	
	return results;
end

function getEffectsBonusByType(rActor, aEffectType, bAddEmptyBonus, aFilter, rFilterActor, bTargetedOnly)
	if not rActor or not aEffectType then
		return {}, 0;
	end
	
	-- MAKE BONUS TYPE INTO TABLE, IF NEEDED
	if type(aEffectType) ~= "table" then
		aEffectType = { aEffectType };
	end
	
	-- PER EFFECT TYPE VARIABLES
	local results = {};
	local bonuses = {};
	local penalties = {};
	local nEffectCount = 0;
	
	for k, v in pairs(aEffectType) do
		-- LOOK FOR EFFECTS THAT MATCH BONUSTYPE
		local aEffectsByType = getEffectsByType(rActor, v, aFilter, rFilterActor, bTargetedOnly);

		-- ITERATE THROUGH EFFECTS THAT MATCHED
		for k2,v2 in pairs(aEffectsByType) do
			-- LOOK FOR ENERGY OR BONUS TYPES
			local dmg_type = nil;
			for _,v3 in pairs(v2.remainder) do
				if StringManager.contains(GameSystem.dmgtypes, v3) or StringManager.contains(GameSystem.immunetypes, v3) then
					dmg_type = v3;
					break;
				end
			end
			
			-- ADD EFFECT RESULTS 
			local new_key = dmg_type or "";
			local new_results = results[new_key] or {dice = {}, mod = 0, remainder = {}};

			-- BUILD THE NEW RESULT
			for _,v3 in pairs(v2.dice) do
				table.insert(new_results.dice, v3); 
			end
			if bAddEmptyBonus then
				new_results.mod = new_results.mod + v2.mod;
			else
				new_results.mod = math.max(new_results.mod, v2.mod);
			end
			for _,v3 in pairs(v2.remainder) do
				table.insert(new_results.remainder, v3);
			end

			-- SET THE NEW DICE RESULTS BASED ON ENERGY TYPE
			results[new_key] = new_results;
			
			-- INCREMENT EFFECT COUNT
			nEffectCount = nEffectCount + 1;
		end
	end

	-- COMBINE BONUSES AND PENALTIES FOR NON-ENERGY TYPED MODIFIERS
	for k2,v2 in pairs(bonuses) do
		if results[k2] then
			results[k2].mod = results[k2].mod + v2;
		else
			results[k2] = {dice = {}, mod = v2, remainder = {}};
		end
	end
	for k2,v2 in pairs(penalties) do
		if results[k2] then
			results[k2].mod = results[k2].mod + v2;
		else
			results[k2] = {dice = {}, mod = v2, remainder = {}};
		end
	end

	return results, nEffectCount;
end

function getEffectsBonus(rActor, aEffectType, bModOnly, aFilter, rFilterActor, bTargetedOnly)
	if not rActor or not aEffectType then
		if bModOnly then
			return 0, 0;
		end
		return {}, 0, 0;
	end
	
	-- MAKE BONUS TYPE INTO TABLE, IF NEEDED
	if type(aEffectType) ~= "table" then
		aEffectType = { aEffectType };
	end
	
	-- START WITH AN EMPTY MODIFIER TOTAL
	local aTotalDice = {};
	local nTotalMod = 0;
	local nEffectCount = 0;
	
	-- ITERATE THROUGH EACH BONUS TYPE
	local masterbonuses = {};
	local masterpenalties = {};
	for k, v in pairs(aEffectType) do
		-- GET THE MODIFIERS FOR THIS MODIFIER TYPE
		local effbonusbytype, nEffectSubCount = getEffectsBonusByType(rActor, v, true, aFilter, rFilterActor, bTargetedOnly);
		
		-- ITERATE THROUGH THE MODIFIERS
		for k2, v2 in pairs(effbonusbytype) do
			for k3, v3 in pairs(v2.dice) do
				table.insert(aTotalDice, v3);
			end
			nTotalMod = nTotalMod + v2.mod;
		end

		-- ADD TO EFFECT COUNT
		nEffectCount = nEffectCount + nEffectSubCount;
	end

	-- ADD INTEGRATED BONUSES AND PENALTIES FOR NON-ENERGY TYPED MODIFIERS
	for k,v in pairs(masterbonuses) do
		nTotalMod = nTotalMod + v;
	end
	for k,v in pairs(masterpenalties) do
		nTotalMod = nTotalMod + v;
	end
	
	-- RESULTS
	if bModOnly then
		return nTotalMod, nEffectCount;
	end
	return aTotalDice, nTotalMod, nEffectCount;
end

function isEffectTarget(nodeEffect, rTarget)
	local bMatch = false;
	
	local sTargetCT = ActorManager.getCTNodeName(rTarget);
	if sTargetCT ~= "" then
		for _,v in pairs(DB.getChildren(nodeEffect, "targets")) do
			if DB.getValue(v, "noderef", "") == sTargetCT then
				bMatch = true;
				break;
			end
		end
	end

	return bMatch;
end

function hasEffectCondition(rActor, sEffect)
	return hasEffect(rActor, sEffect, nil, false, true);
end

function hasEffect(rActor, sEffect, rTarget, bTargetedOnly, bIgnoreEffectTargets)
	if not sEffect or not rActor then
		return false;
	end
	local sLowerEffect = sEffect:lower();
	
	-- Iterate through each effect
	local aMatch = {};
	for _,v in pairs(DB.getChildren(ActorManager.getCTNode(rActor), "effects")) do
		local nActive = DB.getValue(v, "isactive", 0);
		if nActive ~= 0 then
			-- Parse each effect label
			local sLabel = DB.getValue(v, "label", "");
			local bTargeted = EffectManager.isTargetedEffect(v);
			local effect_list = EffectManager.parseEffect(sLabel);

			-- Iterate through each effect component looking for a type match
			local nMatch = 0;
			for kEffectComp, rEffectComp in ipairs(effect_list) do
				-- CHECK FOR AN ACTUAL EFFECT MATCH
				if string.lower(rEffectComp.original) == sLowerEffect then
					if bTargeted and not bIgnoreEffectTargets then
						if isEffectTarget(v, rTarget) then
							nMatch = kEffectComp;
						end
					elseif not bTargetedOnly then
						nMatch = kEffectComp;
					end
				end
				
				if nMatch > 0 then
					table.insert(aMatch, v);
				end
			end
		end
	end
	
	-- Return results
	if #aMatch > 0 then
		return true;
	end
	return false;
end

--
--  HANDLE EFFECT LOCKING
--

function lock()
	nLocked = nLocked + 1;
end

function unlock()
	nLocked = nLocked - 1;
	if nLocked < 0 then
		nLocked = 0;
	end
end

--
-- EFFECT TARGETING
--

function addEffectTarget(vEffect, sTargetNode)
	local nodeTargetList = nil;
	if type(vEffect) == "string" then
		nodeTargetList = DB.createChild(DB.findNode(vEffect), "targets");
	elseif type(vEffect) == "databasenode" then
		nodeTargetList = DB.createChild(vEffect, "targets");
	end
	if not nodeTargetList then
		return;
	end
	
	for _,nodeTarget in pairs(nodeTargetList.getChildren()) do
		if (DB.getValue(nodeTarget, "noderef", "") == sTargetNode) then
			return;
		end
	end

	local nodeNewTarget = nodeTargetList.createChild();
	if nodeNewTarget then
		DB.setValue(nodeNewTarget, "noderef", "string", sTargetNode);
	end
end

function setEffectFactionTargets(nodeEffect, sFaction, bNegated)
	if not nodeEffect then
		return;
	end
	
	clearEffectTargets(nodeEffect);
	
	for _,nodeCT in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		if bNegated then
			if DB.getValue(nodeCT, "friendfoe", "") ~= sFaction then
				addEffectTarget(nodeEffect, nodeCT.getNodeName());
			end
		else
			if DB.getValue(nodeCT, "friendfoe", "") == sFaction then
				addEffectTarget(nodeEffect, nodeCT.getNodeName());
			end
		end
	end
end

function clearEffectTargets(nodeEffect)
	for _,nodeTarget in pairs(DB.getChildren(nodeEffect, "targets")) do
		nodeTarget.delete();
	end
end
