-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

-- Reset mana and individual spells cast
function resetSpells(nodeCaster)
	local nodeSpellClass = nodeCaster.getChild("spellset.set1");
	
	DB.setValue(nodeSpellClass, "mana.used", "number", 0);
	
	for _,nodeLevel in pairs(DB.getChildren(nodeSpellClass, "levels")) do
		for _,nodeSpell in pairs(DB.getChildren(nodeLevel, "spells")) do
			DB.setValue(nodeSpell, "cast", "number", 0);
		end
	end
end

-- Iterate through each spell to reset
function resetPrepared(nodeCaster)
	local nodeSpellClass = nodeCaster.getChild("spellset.set1");

	for _,nodeLevel in pairs(DB.getChildren(nodeSpellClass, "levels")) do
		for _,nodeSpell in pairs(DB.getChildren(nodeLevel, "spells")) do
			DB.setValue(nodeSpell, "prepared", "number", 0);
		end
	end
end

function addSpell(nodeSource, nodeTargetLevelSpells)
	if not nodeSource or not nodeTargetLevelSpells then
		return nil;
	end
	
	local nodeNewSpell = nodeTargetLevelSpells.createChild();
	if not nodeNewSpell then
		return nil;
	end
	
	-- Copy the spell details over
	DB.copyNode(nodeSource, nodeNewSpell);
	
	-- Other checks
	local nodeParent = nodeTargetLevelSpells.getParent();
	if nodeParent then
		-- Set the default cost for mana casters
		local nCost = tonumber(string.sub(nodeParent.getName(), -1)) or 0;
		nCost = nCost + 1;
		DB.setValue(nodeNewSpell, "cost", "number", nCost);

		-- If spell level not visible, then make it so.
		local sAvailablePath = "....spellsavailable.available" .. nodeParent.getName();
		local nAvailable = DB.getValue(nodeTargetLevelSpells, sAvailablePath, 1);
		if nAvailable <= 0 then
			DB.setValue(nodeTargetLevelSpells, sAvailablePath, "number", 1);
		end
	end
	
	return nodeNewSpell;
end

-- Caster Types: 0 = Memorization, 1 = Spontaneous, 2 = Mana
function updateSpellClassCounts(nodeSpellClass)
	local nCasterType = DB.getValue(nodeSpellClass, "spontaneous", 0);
	if nCasterType == 2 then
		return;
	end
	
	for _,vLevel in pairs(DB.getChildren(nodeSpellClass, "levels")) do
		-- Calculate spell statistics
		local nTotalCast = 0;
		local nTotalPrepared = 0;
		local nMaxPrepared = 0;
		local nSpells = 0;
		
		for _,vSpell in pairs(DB.getChildren(vLevel, "spells")) do
			nSpells = nSpells + 1;
			
			local nCast = DB.getValue(vSpell, "cast", 0);
			nTotalCast = nTotalCast + nCast;
			
			local nPrepared = 0;
			if nCasterType == 0 then
				nPrepared = DB.getValue(vSpell, "prepared", 0);
				nTotalPrepared = nTotalPrepared + nPrepared;
				if nPrepared > nMaxPrepared then
					nMaxPrepared = nPrepared;
				end
			end
		end
		
		DB.setValue(vLevel, "totalcast", "number", nTotalCast);
		DB.setValue(vLevel, "totalprepared", "number", nTotalPrepared);
		DB.setValue(vLevel, "maxprepared", "number", nMaxPrepared);
	end
end
