-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

local TOKEN_MAX_EFFECTS = 6;
local TOKEN_EFFECT_WIDTH = 12;
local TOKEN_EFFECT_MARGIN = 2;
local TOKEN_EFFECT_OFFSETX = 6;
local TOKEN_EFFECT_OFFSETY = -6;
local TOKEN_HEALTH_MINBAR = 14;
local TOKEN_HEALTH_WIDTH = 20;

function onInit()
	DB.addHandler("combattracker.list.*.hp", "onUpdate", updateHealth);
	DB.addHandler("combattracker.list.*.hptemp", "onUpdate", updateHealth);
	DB.addHandler("combattracker.list.*.nonlethal", "onUpdate", updateHealth);
	DB.addHandler("combattracker.list.*.wounds", "onUpdate", updateHealth);

	DB.addHandler("options.TNPCH", "onUpdate", TokenManager.onOptionChanged);
	DB.addHandler("options.TPCH", "onUpdate", TokenManager.onOptionChanged);
	DB.addHandler("options.WNDC", "onUpdate", TokenManager.onOptionChanged);
end

function onScaleChanged(tokenCT, nodeCT)
	updateHealthBarScale(tokenCT, nodeCT);
end

function onHover(tokenCT, nodeCT, bOver)
	local sFaction = DB.getValue(nodeCT, "friendfoe", "");

	local sOptHealth;
	if sFaction == "friend" then
		sOptHealth = OptionsManager.getOption("TPCH");
	else
		sOptHealth = OptionsManager.getOption("TNPCH");
	end
		
	local aWidgets = {};
	if sOptHealth == "barhover" then
		aWidgets["healthbar"] = tokenCT.findWidget("healthbar");
	elseif sOptHealth == "dothover" then
		aWidgets["healthdot"] = tokenCT.findWidget("healthdot");
	end

	for _, vWidget in pairs(aWidgets) do
		vWidget.setVisible(bOver);
	end
end

function updateAttributesHelper(tokenCT, nodeCT)
	updateHealthHelper(tokenCT, nodeCT);
end

function updateTooltip(tokenCT, nodeCT)
	local sOptTNAM = OptionsManager.getOption("TNAM");
	local sOptTH;
	if DB.getValue(nodeCT, "friendfoe", "") == "friend" then
		sOptTH = OptionsManager.getOption("TPCH");
	else
		sOptTH = OptionsManager.getOption("TNPCH");
	end
		
	local aTooltip = {};
	if sOptTNAM == "tooltip" then
		table.insert(aTooltip, DB.getValue(nodeCT, "name", ""));
	end
	if sOptTH == "tooltip" then
		table.insert(aTooltip, ActorManager2.getHealth("ct", nodeCT):status());
	end
	
	tokenCT.setName(table.concat(aTooltip, "\r"));
end

function updateFaction(tokenCT, nodeCT)
	updateHealthHelper(tokenCT, nodeCT);
end

function updateHealth(nodeField)
	local nodeCT = nodeField.getParent();
	local tokenCT = CombatManager.getTokenFromCT(nodeCT);
	if tokenCT then
		updateHealthHelper(tokenCT, nodeCT);
		updateTooltip(tokenCT, nodeCT);
	end
end

function updateHealthHelper(tokenCT, nodeCT)
	local sOptTH;
	if DB.getValue(nodeCT, "friendfoe", "") == "friend" then
		sOptTH = OptionsManager.getOption("TPCH");
	else
		sOptTH = OptionsManager.getOption("TNPCH");
	end
	
	local aWidgets = getWidgetList(tokenCT, "health");
	
	if sOptTH == "off" or sOptTH == "tooltip" then
		for _, vWidget in pairs(aWidgets) do
			vWidget.destroy();
		end
	else
		local oHealth = ActorManager2.getHealth("ct", nodeCT);

		if sOptTH == "bar" or sOptTH == "barhover" then
			local w, h = tokenCT.getSize();
		
			if h >= TOKEN_HEALTH_MINBAR then
				local widgetHealthBar = aWidgets["healthbar"];
				if not widgetHealthBar then
					widgetHealthBar = tokenCT.addBitmapWidget("healthbar");
					widgetHealthBar.sendToBack();
					widgetHealthBar.setName("healthbar");
				end
				if widgetHealthBar then
					widgetHealthBar.setColor(oHealth:woundBarColor());
					widgetHealthBar.setTooltipText(oHealth:status());
					widgetHealthBar.setVisible(sOptTH == "bar");
				end
			end
			updateHealthBarScale(tokenCT, nodeCT);
			
			if aWidgets["healthdot"] then
				aWidgets["healthdot"].destroy();
			end
		elseif sOptTH == "dot" or sOptTH == "dothover" then
			local widgetHealthDot = aWidgets["healthdot"];
			if not widgetHealthDot then
				widgetHealthDot = tokenCT.addBitmapWidget("healthdot");
				widgetHealthDot.setPosition("bottomright", -4, -6);
				widgetHealthDot.setName("healthdot");
			end
			if widgetHealthDot then
				widgetHealthDot.setColor(oHealth:woundBarColor());
				widgetHealthDot.setTooltipText(oHealth:status());
				widgetHealthDot.setVisible(sOptTH == "dot");
			end

			if aWidgets["healthbar"] then
				aWidgets["healthbar"].destroy();
			end
		end
	end
end

function updateHealthBarScale(tokenCT, nodeCT)
	local widgetHealthBar = tokenCT.findWidget("healthbar");
	if widgetHealthBar then
		local oHealth = ActorManager2.getHealth("ct", nodeCT);
		
		local w, h = tokenCT.getSize();
		h = h + 4;

		widgetHealthBar.setSize();
		local barw, barh = widgetHealthBar.getSize();
		
		-- Resize bar to match health percentage, but preserve bulb portion of bar graphic
		if h >= TOKEN_HEALTH_MINBAR then
			barh = (math.max(1.0 - oHealth:percentDamaged(), 0) * (math.min(h, barh) - TOKEN_HEALTH_MINBAR)) + TOKEN_HEALTH_MINBAR;
		else
			barh = TOKEN_HEALTH_MINBAR;
		end

		widgetHealthBar.setSize(barw, barh, "bottom");
		widgetHealthBar.setPosition("bottomright", -4, -(barh / 2) + 4);
	end
end

function getWidgetList(tokenCT, sSubset)
	local aWidgets = {};

	local w = nil;
	if not sSubset or sSubset == "health" then
		for _, vName in pairs({"healthbar", "healthdot"}) do
			w = tokenCT.findWidget(vName);
			if w then
				aWidgets[vName] = w;
			end
		end
	end
	
	return aWidgets;
end
