-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

local rsname = "Castles and Crusades";
local rsmajorversion = 14;
local aMigrated = {};

function onInit()
	if User.isHost() or User.isLocal() then
		updateCampaign();
	end

	DB.onAuxCharLoad = onCharImport;
	DB.onImport = onImport;
	Module.onModuleLoad = onModuleLoad;
end

function onCharImport(nodePC)
	local _, _, aMajor, _ = DB.getImportRulesetVersion();
	updateChar(nodePC, aMajor[rsname]);
end

function onImport(node)
	local aPath = StringManager.split(node.getNodeName(), ".");
	if #aPath == 2 and aPath[1] == "charsheet" then
		local _, _, aMajor, _ = DB.getImportRulesetVersion();
		updateChar(node, aMajor[rsname]);
	end
end

function onModuleLoad(sModule)
	local _, _, aMajor, _ = DB.getRulesetVersion(sModule);
	updateModule(sModule, aMajor[rsname]);
end

function updateChar(nodePC, nVersion)
	if not nVersion then
		nVersion = 0;
	end
	
	if nVersion < rsmajorversion then
		if nVersion < 2 then
			migrateChar2(nodePC);
		end
		if nVersion < 11 then
			migrateChar11(nodePC);
		end
		if nVersion < 12 then
			migrateChar12(nodePC);
		end
	end
end

function updateCampaign()
	local _, _, aMajor, aMinor = DB.getRulesetVersion();
	local major = aMajor[rsname];
	if not major then
		return;
	end
	
	if major > 0 and major < rsmajorversion then
		print("Migrating campaign database to latest data version.");
		DB.backup();
		
		if major < 2 then
			convertCharacters2();
		end
		if major < 11 then
			convertCharacters11();
			convertCombat11();
		end
		if major < 12 then
			convertCharacters12();
		end
		if major < 13 then
			convertPSEnc13();
		end
		if major < 14 then
			convertNotes14();
			-- convertImage14(); // Can't fix image links, because no setValue function
			convertTables14();
			convertStories14();
			convertQuests14();
			convertEncounters14();
		end
	end
end

function updateModule(sModule, nVersion)
	if not nVersion then
		nVersion = 0;
	end
	
	if nVersion < rsmajorversion then
		if nVersion < 14 then
			convertModuleTables14(nodeRoot);
			convertModuleStories14(nodeRoot);
			convertModuleQuests14(nodeRoot);
			convertModuleEncounters14(nodeRoot);
		end
	end
end

-- local aPathUpdates = {
	-- -- TLG8505FG2
	-- { sPathReplace = "magictreasuredescription\.", sPathTarget = "reference\.magicitems\." },
	-- { sPathReplace = "monsterdescription\.index\.", sPathTarget = "reference\.npcdata\." },
	-- -- TLG8502FG2
	-- { sPathReplace = "armsnarmordescription\.", sPathTarget = "reference\.equipment\." },		*** !!
	-- -- TLG83201FG2
	-- { sPathReplace = "reference\.newmonsters\.", sPathTarget = "reference\.npcdata\." },			*** !!
	-- -- TLG8125FG2
	-- { sPathReplace = "monsterdescription\.index\.", sPathTarget = "reference\.npcdata\." },
	-- -- TLG8019FG2
	-- { sPathReplace = "magictreasuredescription\.", sPathTarget = "reference\.magicitems\." },
	-- { sPathReplace = "monsterdescription\.", sPathTarget = "reference\.npcdata\." },				*** !!
	-- -- TLG8017FG2
	-- { sPathReplace = "monsters\.[^\.]+\.index\.", sPathTarget = "reference\.npcdata\." },		*** !!
	-- -- DGA055
	-- { sPathReplace = "monsterdescription\.index\.", sPathTarget = "reference\.npcdata\." },		*** !!
	-- { sPathReplace = "monsterdescription2\.index\.", sPathTarget = "reference\.npcdata\." },		*** !!
	-- { sPathReplace = "monsterdescription3\.index\.", sPathTarget = "reference\.npcdata\." },		*** !!
	-- -- DGA045
	-- { sPathReplace = "magictreasuredescription\.", sPathTarget = "reference\.magicitems\." },	*** !!
	-- { sPathReplace = "monsterdescription\.index\.", sPathTarget = "reference\.npcdata\." },		
-- };
local aPathUpdates14 = {
	{ sPathReplace = "magictreasuredescription\.", sPathTarget = "reference\.magicitems\." },
	{ sPathReplace = "monsterdescription\.index\.", sPathTarget = "reference\.monsterdata\." },
	{ sPathReplace = "monsterdescription2\.index\.", sPathTarget = "reference\.monsterdata\." },
	{ sPathReplace = "monsterdescription3\.index\.", sPathTarget = "reference\.monsterdata\." },
	{ sPathReplace = "armsnarmordescription\.", sPathTarget = "reference\.equipment\." },
	{ sPathReplace = "reference\.newmonsters\.", sPathTarget = "reference\.monsterdata\." },
	{ sPathReplace = "monsterdescription\.", sPathTarget = "reference\.monsterdata\." },
	{ sPathReplace = "monsters\.[^\.]+\.index\.", sPathTarget = "reference\.monsterdata\." },
};

function convertNotes14()
	for _,vNote in pairs(DB.getChildren("notes")) do
		convertFTEntry14(vNote, "text");
	end
end

function convertStories14()
	for _,vStory in pairs(DB.getChildren("encounter")) do
		convertFTEntry14(vStory, "text");
	end
end

function convertModuleStories14(nodeRoot)
	for _,vStory in pairs(DB.getChildren(nodeRoot, "encounter")) do
		convertFTEntry14(vStory, "text");
	end
end

function convertQuests14()
	for _,vQuest in pairs(DB.getChildren("quest")) do
		convertFTEntry14(vQuest, "description");
	end
end

function convertModuleQuests14(nodeRoot)
	for _,vQuest in pairs(DB.getChildren(nodeRoot, "quest")) do
		convertFTEntry14(vQuest, "description");
	end
end

function convertFTEntry14(vNode, sField)
	local vText = DB.getChild(vNode, sField);
	if vText then
		local sFT = vText.getValue();
		for k,v in ipairs(aPathUpdates14) do
			sFT = sFT:gsub("recordname=\"" .. v.sPathReplace, "recordname=\"" .. v.sPathTarget);
		end
		vText.setValue(sFT);
	end
end

function convertTables14()
	for _,vTable in pairs(DB.getChildren("tables")) do
		convertTable14(vTable);
	end
end

function convertModuleTables14(nodeRoot)
	for _,vTable in pairs(DB.getChildren(nodeRoot, "tables")) do
		convertTable14(vTable);
	end
end

function convertTable14(vTable)
	local vRows = DB.getChild(vTable, "tablerows");
	if vRows then
		for _,vRow in pairs(vRows.getChildren()) do
			local vResults = DB.getChild(vRow, "results");
			if vResults then
				for _,vResult in pairs(vResults.getChildren()) do
					local sClass, sRecord = DB.getValue(vResult, "resultlink", "", "");
					for k,v in ipairs(aPathUpdates14) do
						sRecord = sRecord:gsub(v.sPathReplace, v.sPathTarget);
					end
					DB.setValue(vResult, "resultlink", "windowreference", sClass, sRecord);
				end
			end
		end
	end
end

function convertEncounters14()
	for _,vEncounter in pairs(DB.getChildren("battle")) do
		convertEncounter14(vEncounter);
	end
end

function convertModuleEncounters14(nodeRoot)
	for _,vEncounter in pairs(DB.getChildren(nodeRoot, "battle")) do
		convertEncounter14(vEncounter);
	end
end

function convertEncounter14(vEncounter)
	local vNPCs = DB.getChild(vEncounter, "npclist");
	if vNPCs then
		for _,vNPC in pairs(vNPCs.getChildren()) do
			local sClass, sRecord = DB.getValue(vNPC, "link", "", "");
			for k,v in ipairs(aPathUpdates14) do
				sRecord = sRecord:gsub(v.sPathReplace, v.sPathTarget);
			end
			DB.setValue(vNPC, "link", "windowreference", sClass, sRecord);
		end
	end
end

function convertPSEnc13()
	for _,vEnc in pairs(DB.getChildren("partysheet.encounters")) do
		DB.setValue(vEnc, "exp", "number", DB.getValue(vEnc, "xp", "number"));
	end
end

function migrateChar12(vPC)
	local nodeAbilityList = DB.createChild(vPC, "abilitylist");
	if nodeAbilityList then
		for _,vFeat in pairs(DB.getChildren(vPC, "featlist")) do
			local nodeAbility = DB.createChild(nodeAbilityList);
			if nodeAbility then
				DB.copyNode(vFeat, nodeAbility);
				vFeat.delete();
			end
		end
		if DB.getChildCount(vPC, "featlist") == 0 then
			DB.deleteChild(vPC, "featlist");
		end
		
		for _,vSA in pairs(DB.getChildren(vPC, "specialabilitylist")) do
			local nodeAbility = DB.createChild(nodeAbilityList);
			if nodeAbility then
				DB.copyNode(vSA, nodeAbility);
				vSA.delete();
			end
		end
		if DB.getChildCount(vPC, "specialabilitylist") == 0 then
			DB.deleteChild(vPC, "specialabilitylist");
		end
	end
end

function convertCharacters12()
	for _,vPC in pairs(DB.getChildren("charsheet")) do
		migrateChar12(vPC);
	end
end

function convertCombat11()
	if DB.findNode("combattracker.list") then
		return;
	end
	
	local aCombatants = DB.getChildren("combattracker");
	local nodeList = DB.createNode("combattracker.list");
	
	for _,vNPC in pairs(aCombatants) do
		local nodeNew = nodeList.createChild();

		-- Get rid of type field
		DB.deleteChild(vNPC, "type");
		
		-- Migrate show_npc to tokenvis
		DB.setValue(vNPC, "tokenvis", "number", DB.getValue(vNPC, "show_npc", 0));
		DB.deleteChild(vNPC, "show_npc");
		
		-- Migrate speed to move
		DB.setValue(vNPC, "monstermove", "string", DB.getValue(vNPC, "speed", ""));
		DB.deleteChild(vNPC, "speed");
		
		-- Parse AC
		local nAC = DB.getValue(vNPC, "ac", 10);
		DB.setValue(vNPC, "ac_final", "number", nAC);
		DB.setValue(vNPC, "ac_flatfooted", "number", nAC);
		DB.setValue(vNPC, "ac_touch", "number", 10);
		
		-- Copy node over to new location
		-- And convert partial to full records
		local sClass, sRecord = DB.getValue(vNPC, "link", "", "");
		if sRecord == "" or sClass == "charsheet" then
			DB.copyNode(vNPC, nodeNew);
			if sClass ~= "charsheet" then
				DB.setValue(nodeNew, "link", "windowreference", "npc", "");
			end
		else
			local nodeRecord = DB.findNode(sRecord);
			if nodeRecord then
				DB.copyNode(nodeRecord, nodeNew);
			end
			DB.copyNode(vNPC, nodeNew);
			DB.setValue(nodeNew, "link", "windowreference", "npc", "");
		end
		
		-- Delete the old record
		vNPC.delete();
	end
	
	-- Migrate round counter to new location
	DB.setValue("combattracker.round", "number", DB.getValue("combattracker_props.round", 0));
	DB.deleteNode("combattracker_props");
end

function migrateChar11(vPC)
	local nLevel = tonumber(DB.getValue(vPC, "classes.slot1.level", "0")) or 0;
	DB.deleteChild(vPC, "classes.slot1.level");
	DB.setValue(vPC, "classes.slot1.level", "number", nLevel);
	
	DB.deleteChild(vPC, "classes.slot2");
	DB.deleteChild(vPC, "classes.slot3");
	DB.deleteChild(vPC, "spellset.set2");
	DB.deleteChild(vPC, "spellset.set3");
	
	DB.deleteChild(vPC, "abilities.strength.damage");
	DB.deleteChild(vPC, "abilities.dexterity.damage");
	DB.deleteChild(vPC, "abilities.constitution.damage");
	DB.deleteChild(vPC, "abilities.intelligence.damage");
	DB.deleteChild(vPC, "abilities.wisdom.damage");
	DB.deleteChild(vPC, "abilities.charisma.damage");
	
	DB.setValue(vPC, "speed.base", "number", DB.getValue(vPC, "move", 0));
	DB.deleteChild(vPC, "move");
	
	if DB.findNode("special.spellresistance") then
		DB.setValue(vPC, "defenses.sr.base", "number", DB.getValue(vPC, "special.spellresistance", 0));
		DB.deleteChild(vPC, "special.spellresistance");
	end
	
	DB.deleteChild(vPC, "encumbranceheavyload");
	DB.deleteChild(vPC, "encumbrancelightload");
	DB.deleteChild(vPC, "encumbrancemoderateload");
	DB.deleteChild(vPC, "encumbrancenone");
	DB.deleteChild(vPC, "encumbranceoverburdenedload");
end

function convertCharacters11()
	for _,vPC in pairs(DB.getChildren("charsheet")) do
		migrateChar11(vPC);
	end
end

function migrateChar2(vPC)
	checkInventory2(vPC);
	checkSpecialAbilities2(vPC);
	convertFields2(vPC);
end

function convertCharacters2()
	for _,vPC in pairs(DB.getChildren("charsheet")) do
		migrateChar(vPC);
	end
end

function checkInventory2(nodePC)
	local nodeNewInventory = nodePC.createChild("inventorylist");
	if nodeNewInventory then
		local nodeOldInventory = nodePC.getChild("inventory");
		if nodeOldInventory then
			for _,vOldItem in pairs(nodeOldInventory.getChildren()) do
				local nodeNewItem = nodeNewInventory.createChild();
				DB.copyNode(vOldItem, nodeNewItem);
			end
			nodeOldInventory.delete();
		end
		
		local nodeOldEquipped = nodePC.getChild("equippedinventory");
		if nodeOldEquipped then
			for _,vOldItem in pairs(nodeOldEquipped.getChildren()) do
				local nodeNewItem = nodeNewInventory.createChild();
				DB.copyNode(vOldItem, nodeNewItem);
			end
			nodeOldEquipped.delete();
		end
	end
end

function checkSpecialAbilities2(nodePC)
	local nodeNewSpecial = nodePC.createChild("specialabilitylist");
	if nodeNewSpecial then
		local nodeOldSpecial = nodePC.getChild("specialabilities");
		if nodeOldSpecial then
			for _,vOldSpecial in pairs(nodeOldSpecial.getChildren()) do
				local nodeNewSpecial = nodeNewSpecial.createChild();
				DB.copyNode(vOldSpecial, nodeNewSpecial);
			end
			nodeOldSpecial.delete();
		end
	end
end

local aFieldConversions2 = {
	["strength"] = "abilities.strength.score",
	["strengthdamage"] = "abilities.strength.damage",
	["strengthbonus"] = "abilities.strength.bonus",
	["strengthbonusmodifier"] = "abilities.strength.bonusmodifier",
	["strengthprime"] = "abilities.strength.prime",
	["dexterity"] = "abilities.dexterity.score",
	["dexteritydamage"] = "abilities.dexterity.damage",
	["dexteritybonus"] = "abilities.dexterity.bonus",
	["dexteritybonusmodifier"] = "abilities.dexterity.bonusmodifier",
	["dexterityprime"] = "abilities.dexterity.prime",
	["constitution"] = "abilities.constitution.score",
	["constitutiondamage"] = "abilities.constitution.damage",
	["constitutionbonus"] = "abilities.constitution.bonus",
	["constitutionbonusmodifier"] = "abilities.constitution.bonusmodifier",
	["constitutionprime"] = "abilities.constitution.prime",
	["intelligence"] = "abilities.intelligence.score",
	["intelligencedamage"] = "abilities.intelligence.damage",
	["intelligencebonus"] = "abilities.intelligence.bonus",
	["intelligencebonusmodifier"] = "abilities.intelligence.bonusmodifier",
	["intelligenceprime"] = "abilities.intelligence.prime",
	["wisdom"] = "abilities.wisdom.score",
	["wisdomdamage"] = "abilities.wisdom.damage",
	["wisdombonus"] = "abilities.wisdom.bonus",
	["wisdombonusmodifier"] = "abilities.wisdom.bonusmodifier",
	["wisdomprime"] = "abilities.wisdom.prime",
	["charisma"] = "abilities.charisma.score",
	["charismadamage"] = "abilities.charisma.damage",
	["charismabonus"] = "abilities.charisma.bonus",
	["charismabonusmodifier"] = "abilities.charisma.bonusmodifier",
	["charismaprime"] = "abilities.charisma.prime",
	
	["class1"] = "classes.slot1.name",
	["level1"] = "classes.slot1.level",
	["class2"] = "classes.slot2.name",
	["level2"] = "classes.slot2.level",
	["class3"] = "classes.slot3.name",
	["level3"] = "classes.slot3.level",
	
	["hp"] = "hp.total",
	["wounds"] = "hp.wounds",
	["subdual"] = "hp.nonlethal",
	
	["ac"] = "ac.totals.general",
	["acmodifier"] = "ac.sources.temporary",
	["acarmorbonus"] = "ac.sources.armor",
	["acshieldbonus"] = "ac.sources.shield",
	["acsizebonus"] = "ac.sources.size",
	["acnaturalarmorbonus"] = "ac.sources.naturalarmor",
	["acdeflectionbonus"] = "ac.sources.deflection",
	["acmiscbonus"] = "ac.sources.misc",
	["touchac"] = "ac.totals.touch",
	
	["initiative"] = "initiative.total",
	["initiativemiscbonus"] = "initiative.misc",
	
	["baseattackbonus"] = "attackbonus.base",
	["meleeattackbonus"] = "attackbonus.melee",
	["rangeattackbonus"] = "attackbonus.ranged",
	["grapplebonus"] = "attackbonus.grapple.total",
	["grapplesizebonus"] = "attackbonus.grapple.size",
	["grapplemiscbonus"] = "attackbonus.grapple.misc",
	
	["speed"] = "speed",
	["spellfailure"] = "special.spellfailure",
	["damagereduction"] = "special.damagereduction",
	["spellresistance"] = "special.spellresistance",
	
	["encumbranceload"] = "encumbrance.load",
	["encumbrancerating"] = "encumbrance.rating",
	["encumbrancenone"] = "encumbrance.none",
	["encumbrancelightload"] = "encumbrance.lightload",
	["encumbrancemoderateload"] = "encumbrance.moderateload",
	["encumbranceheavyload"] = "encumbrance.heavyload",
	["encumbranceoverburdenedload"] = "encumbrance.overburdenedload",
	
	["coinname1"] = "coins.slot1.name",
	["coinamount1"] = "coins.slot1.amount",
	["coinname2"] = "coins.slot2.name",
	["coinamount2"] = "coins.slot2.amount",
	["coinname3"] = "coins.slot3.name",
	["coinamount3"] = "coins.slot3.amount",
	["coinname4"] = "coins.slot4.name",
	["coinamount4"] = "coins.slot4.amount",
	["coinname5"] = "coins.slot5.name",
	["coinamount5"] = "coins.slot5.amount",
	["coinname6"] = "coins.slot6.name",
	["coinamount6"] = "coins.slot6.amount"
};

function convertFields2(nodePC)
	for sOldField,sNewField in pairs(aFieldConversions2) do
		local nodeOld = nodePC.getChild(sOldField);
		if nodeOld then
			DB.setValue(nodePC, sNewField, nodeOld.getType(), nodeOld.getValue());
			nodeOld.delete();
		end
	end
end
